#ifndef NOISEOCCUPANCYALGORITHM_H
#define NOISEOCCUPANCYALGORITHM_H

#include "AnalysisAlgorithm.h"
class TF1;

namespace SctData {
    class ChipNOResult;
    class NoiseOccupancyTestResult;
    class ChipConfiguration;
    class OccupancyProjector;
    class ModuleDefectList;
}

namespace SctAnalysis {
    
class NoiseOccupancyAlgorithm : public AnalysisAlgorithm {
public:
    /**
     * over-ride
     */
    virtual boost::shared_ptr<AnalysisAlgorithm> clone(const TestData& testData, const string& moduleName) const throw();
    /** 
	constructor 
     */
    NoiseOccupancyAlgorithm(const TestData& testData, const string& moduleName) throw() : AnalysisAlgorithm(testData, moduleName) {}
    /** 
	destructor 
     */
    virtual ~NoiseOccupancyAlgorithm() throw();
    /**
       over-ride -- needs Fits for defects!
	@todo document
    */
    virtual void canAddFitScanResult(const string& name) throw(Sct::LogicError, Sct::IoError);
    /**
       over-ride
       @todo document
    */
    virtual void canAddRawScanResult(const string& name) throw(Sct::LogicError, Sct::IoError);
    /** 
     * given the occupancy projector and response curve, produces result for a chip. 
     */
    static SctData::ChipNOResult analyzeChip(const unsigned ichip, const SctData::OccupancyProjector&, const SctData::ChipConfiguration&, SctData::ModuleDefectList& defects) throw(Sct::LogicError);
private:
    NoiseOccupancyAlgorithm() {}
    void init() throw();                   ///< Creates the new result if necessary.
    /**
       Analysis returns unless all fits and raws are available.
     */
    void analyze() throw(Sct::LogicError);
    static bool inMap;
};
}
#endif //#ifndef NOISEOCCUPANCYALGORITHM_H
