#include "StrobeDelayAlgorithm.h"
#include "AnalysisAlgorithmMap.h"
#include "SctData/StrobeDelayTestResult.h"
#include "SctData/FitScanResult.h"
#include "SctData/FitObject.h"
#include "SctData/TopHatFitObject.h"
#include "SctData/ModuleDefectList.h"
#include "SctData/ModuleDefect.h"
#include <string>

using namespace std;
using namespace boost;
using namespace SctData;
using namespace Sct;

namespace SctAnalysis {
    
bool StrobeDelayAlgorithm::inMap = AnalysisAlgorithmMap::instance().setAlgorithm("StrobeDelayTest", auto_ptr<AnalysisAlgorithm>(new StrobeDelayAlgorithm()));

shared_ptr<AnalysisAlgorithm> StrobeDelayAlgorithm::clone(const TestData& testData, const string& moduleName) const throw(){
    return shared_ptr<AnalysisAlgorithm>(new StrobeDelayAlgorithm(testData, moduleName));
}

void StrobeDelayAlgorithm::canAddFitScanResult(const string& name) throw(Sct::LogicError, Sct::IoError) {
    //Create and initialize
    shared_ptr<StrobeDelayTestResult> result (new StrobeDelayTestResult()) ;
    setTestResult( result );
    initializeTestResult();
    
    addFit(name);
    optimize(*result, *getTestResult()->getFit(0));
    finish();
}

void StrobeDelayAlgorithm::optimize(StrobeDelayTestResult& result, const FitScanResult& fitted) throw(LogicError) {
    
    bool pass = true;
    result.setNOptima(nChipModule);
    for (short unsigned int ichip=0; ichip<nChipModule; ++ichip ) {
        FitObject& fit = fitted.getChipFit(ichip);
        try {
            dynamic_cast<TopHatFitObject&> (fit);
        } catch(std::exception& e) {
            throw InvariantViolatedError("StrobeDelayAlgorithm::optimize ERROR not a TopHat function", __FILE__, __LINE__);
        }
        double rise=fit.getParameter(1), fall=fit.getParameter(3);
        result.setOptimum(ichip, rise + result.getFraction()*(fall-rise));
	if (rise < 0 || rise > ModuleDefect::SD_LO.getParameter()) {
	    result.getDefects().addDefect(ModuleDefect::SD_LO, Chip(ichip));
	    pass = false;
	} 
	if (fall > ModuleDefect::SD_HI.getParameter()) {
	    result.getDefects().addDefect(ModuleDefect::SD_HI, Chip(ichip));
	    pass = false;
	}
    }
    result.setPassed(pass);
}


}
