#ifndef RESPONSECURVE_H
#define RESPONSECURVE_H

#include "../ipc/TestRequest.h"
#include "../ipc/CalibrationController.h"
#include "../ipc/ScanLibrary.h"
#include "../ipc/Sct_CalibrationController.hh"
#include "../CalibrationControllerImpl.h"

#include "../SctApiException.h"
#include "Sct/LogicErrors.h"

#include <CommonWithDsp/ABCD/ABCDscans.h>
#include <iostream>
#include <vector>

using namespace Sct;
using namespace std;
using namespace SctCalibrationController::Ipc;
using namespace SctCalibrationController;

namespace SctCalibrationController {
namespace Scripts {
    
class ResponseCurve : public TestRequestImpl {
public:	
    static ResponseCurve& instance() {
	static ResponseCurve rc;
	return rc;
    }
    
    virtual void startTest(Sct_SctApi_T_SctApi& api, string& name, unsigned short& testVariable, unsigned long& nScans, vector<double>& testPoints) {	
	this->api = &api;
	
	Sct_SctApiStatus status;
	api.modifyABCDVar(&status, ST_COMPRESSION, 1);
	if (status.returnCode != 0)
	    throw SctApiException(status.returnCode, "ResponseCurve::startTest", __FILE__, __LINE__);
	
	api.modifyABCDVar(&status, ST_EDGE_DETECT, 0);
	if (status.returnCode != 0)
	    throw SctApiException(status.returnCode, "ResponseCurve::startTest", __FILE__, __LINE__);
	
	api.modifyABCDVar(&status, ST_SEND_MASK, 0);    // Send mask off
	if (status.returnCode != 0)
	    throw SctApiException(status.returnCode, "ResponseCurve::startTest", __FILE__, __LINE__);
	
	name = "NPtGainTest";
	testVariable = ST_QCAL;	
	
	charges.push_back(0.5);
	charges.push_back(0.75);	
	charges.push_back(1.0);
	charges.push_back(1.25);
	charges.push_back(1.5);
	charges.push_back(2.);
	charges.push_back(3);
	charges.push_back(4);
	charges.push_back(6);
	charges.push_back(8);

	testPoints = charges;
	nScans = testPoints.size();
    }
    
    virtual Sct_SctApi_T_Scan* getNextScan(unsigned long index) {
	if (api == 0) 
	    throw InvariantViolatedError("Api pointer is 0 in getNextScan!", __FILE__, __LINE__);
	
	ScanLibrary& sl = CalibrationControllerImpl::instance().getScanLibrary();
	
	Sct_SctApiStatus status;	
	api->modifyABCDVar(&status, ST_QCAL, charges[index]);
	if (status.returnCode != 0)
	    throw SctApiException(status.returnCode, "ResponseCurve::getNextScan", __FILE__, __LINE__);
	
	return &sl.thresholdScanfC(1000, charges[index]);		
    }
    
    virtual bool canFeedback() {
	return true;
    }  
    
    virtual void endTest() {
	api = 0;
	charges.clear();
    }
    
    
private:
    ResponseCurve() {}
    Sct_SctApi_T_SctApi* api;    
    vector<double> charges;
};
}
}
#endif //RESPONSECURVE_H
