#ifndef SROBEDELAYSCAN_H
#define SROBEDELAYSCAN_H

#include "../ipc/Sct_CalibrationController.hh"

#include "../SctApiException.h"
#include "Sct/LogicErrors.h"

#include <CommonWithDsp/ABCD/ABCDscans.h>

using namespace Sct;

namespace SctCalibrationController {
namespace Scripts {
    
class StrobeDelayScan {
public:
    static Sct_SctApi_T_Scan& strobeDelayScan(Sct_SctApi_T_SctApi& api, unsigned long nTrigs) {	
	Sct_SctApiStatus status;
	Sct_SctApi_T_Scan* s = api.defaultScan(&status);
	if (status.returnCode != 0)
	    throw SctApiException(status.returnCode, "StrobeDelayScan", __FILE__, __LINE__);
	
	Sct_SctApi_T_Trigger* t = s->getTrigger1(&status);
	if (status.returnCode != 0)
	    throw SctApiException(status.returnCode, "StrobeDelayScan", __FILE__, __LINE__);
	
	t->calL1A(&status, 0x82);
	if (status.returnCode != 0)
	    throw SctApiException(status.returnCode, "StrobeDelayScan", __FILE__, __LINE__);
	
	s->setNTrigs(&status, nTrigs);
	if (status.returnCode != 0)
	    throw SctApiException(status.returnCode, "StrobeDelayScan", __FILE__, __LINE__);
	
	s->setOption(&status, Sct_SctApi_T_Scan_ScanOptions_loopCalLine, 1);
	if (status.returnCode != 0)
	    throw SctApiException(status.returnCode, "StrobeDelayScan", __FILE__, __LINE__);
	
	s->configure(&status, ST_STROBE_DELAY, 0, 63, 1);
	if (status.returnCode != 0)
	    throw SctApiException(status.returnCode, "StrobeDelayScan", __FILE__, __LINE__);
	
	return *s;
    }
    
private:
    StrobeDelayScan();
};
    
}
}

#endif //SROBEDELAYSCAN_H
