#ifndef STROBEDELAYTEST_H
#define STROBEDELAYTEST_H

#include "../ipc/TestRequest.h"
#include "../ipc/CalibrationController.h"
#include "../ipc/ScanLibrary.h"
#include "../ipc/Sct_CalibrationController.hh"

#include "../SctApiException.h"
#include "Sct/LogicErrors.h"

#include <CommonWithDsp/ABCD/ABCDscans.h>
#include <iostream>
#include <vector>

using namespace Sct;
using namespace std;
using namespace SctCalibrationController::Ipc;
using namespace SctCalibrationController;

namespace SctCalibrationController {
namespace Scripts {
    
class StrobeDelayTest : public TestRequestImpl {
public:	
    static StrobeDelayTest& instance() {
	static StrobeDelayTest rc;
	return rc;
    }
    
    virtual void startTest(Sct_SctApi_T_SctApi& api, string& name, unsigned short& testVariable, unsigned long& nScans, vector<double>& testPoints) {	
	this->api = &api;
	
	Sct_SctApiStatus status;

	api.modifyABCDVar(&status, ST_QTHR, 2);            // Set threshold to 2fC
	if (status.returnCode != 0)
	    throw SctApiException(status.returnCode, "StrobeDelayTest::startTest", __FILE__, __LINE__);

	api.modifyABCDVar(&status, ST_COMPRESSION, 2);     // Compression mode 2
	if (status.returnCode != 0)
	    throw SctApiException(status.returnCode, "StrobeDelayTest::startTest", __FILE__, __LINE__);
	
	api.modifyABCDVar(&status, ST_EDGE_DETECT, 1);     // Edge detect on
	if (status.returnCode != 0)
	    throw SctApiException(status.returnCode, "StrobeDelayTest::startTest", __FILE__, __LINE__);
	
	api.modifyABCDVar(&status, ST_VCAL, 40);           // 4.00 fC input charge
	if (status.returnCode != 0)
	    throw SctApiException(status.returnCode, "StrobeDelayTest::startTest", __FILE__, __LINE__);
	
	api.modifyABCDVar(&status, ST_SEND_MASK, 0);       // Send mask off
	if (status.returnCode != 0)
	    throw SctApiException(status.returnCode, "StrobeDelayTest::startTest", __FILE__, __LINE__);
	
	name = "StrobeDelayTest";
	testVariable = ST_ELAPSED;
	nScans = 1;
	
	testPoints.push_back(0);
    }
    
    virtual Sct_SctApi_T_Scan* getNextScan(unsigned long index) {
	ScanLibrary& sl = CalibrationControllerImpl::instance().getScanLibrary();
	return &sl.strobeDelayScan(1000);		
    }
    
    virtual bool canFeedback() {
	return true;
    }    
    
    virtual void endTest() {
	api = 0;
    }
    
private:
    StrobeDelayTest() {}
    Sct_SctApi_T_SctApi* api;    
};
}
}
#endif //STROBEDELAYTEST_H
