#ifndef THRESHOLDSCAN_H
#define THRESHOLDSCAN_H

#include "../ipc/Sct_CalibrationController.hh"

#include "../SctApiException.h"
#include "Sct/LogicErrors.h"

#include <CommonWithDsp/ABCD/ABCDscans.h>

using namespace Sct;

namespace SctCalibrationController {
namespace Scripts {
    
class ThresholdScan {
public:
    static Sct_SctApi_T_Scan& thresholdScan(Sct_SctApi_T_SctApi& api, unsigned long nTrigs, double calChargefC = -1) {	
	Sct_SctApiStatus status;
	Sct_SctApi_T_Scan* s = api.defaultScan(&status);
	if (status.returnCode != 0)
	    throw SctApiException(status.returnCode, "thresholdScan", __FILE__, __LINE__);
	
	s->setNTrigs(&status, nTrigs);
	if (status.returnCode != 0)
	    throw SctApiException(status.returnCode, "thresholdScan", __FILE__, __LINE__);
	
	Sct_SctApi_T_Trigger* t = s->getTrigger1(&status);
	if (status.returnCode != 0)
	    throw SctApiException(status.returnCode, "thresholdScan", __FILE__, __LINE__);
	
	t->calL1A(&status, 0x82);
	if (status.returnCode != 0)
	    throw SctApiException(status.returnCode, "thresholdScan", __FILE__, __LINE__);

        if (fabs(calChargefC) > 0.05) {
            s->setOption(&status, Sct_SctApi_T_Scan_ScanOptions_loopCalLine, 1);
            if (status.returnCode != 0)
	        throw SctApiException(status.returnCode, "thresholdScan", __FILE__, __LINE__);
        }

	double lo = 30.0 * (calChargefC - 1);
	double hi = 200.0 + 70.0 * (calChargefC - 1);

	//Only set to lowLimit if we have some cal charge!
	if (calChargefC > 0 && lo < lowLimit || calChargefC < 0) lo = lowLimit;
	else if (lo < 0) lo = 0;
	if (hi > highLimit || calChargefC < 0) hi = highLimit;
        if (hi < lo) hi = lo + 100;

	s->configure(&status, ST_VTHR, lo, hi, 5);
	
	return *s;
    }
    
private:
    ThresholdScan();	
    
    static const double lowLimit = 90;
    static const double highLimit = 640;
};
    
}
}

#endif //THRESHOLDSCAN_H
