#ifndef TIMEWALKTEST_H
#define TIMEWALKTEST_H

#include "../ipc/TestRequest.h"
#include "../ipc/CalibrationController.h"
#include "../ipc/ScanLibrary.h"
#include "../ipc/Sct_CalibrationController.hh"

#include "../SctApiException.h"

#include <CommonWithDsp/ABCD/ABCDscans.h>
#include <iostream>
#include <vector>

using namespace Sct;
using namespace std;
using namespace SctCalibrationController::Ipc;
using namespace SctCalibrationController;

namespace SctCalibrationController {
namespace Scripts {
    
class TimeWalkTest : public TestRequestImpl {
public:	
    static TimeWalkTest& instance() {
	static TimeWalkTest tw;
	return tw;
    }
    
    virtual void startTest(Sct_SctApi_T_SctApi& api, string& name, unsigned short& testVariable, unsigned long& nScans, vector<double>& testPoints) {	
	this->api = &api;
	
	Sct_SctApiStatus status;
	
	api.modifyABCDVar(&status, ST_COMPRESSION, 2);     // Compression mode 2
	if (status.returnCode != 0)
	    throw SctApiException(status.returnCode, "TimeWalkTest::startTest", __FILE__, __LINE__);

	api.modifyABCDVar(&status, ST_EDGE_DETECT, 1);     // Edge detect on
	if (status.returnCode != 0)
	    throw SctApiException(status.returnCode, "TimeWalkTest::startTest", __FILE__, __LINE__);
	
	api.modifyABCDVar(&status, ST_VCAL, 12.5);         // 1 fC input charges
	if (status.returnCode != 0)
	    throw SctApiException(status.returnCode, "TimeWalkTest::startTest", __FILE__, __LINE__);
	
	api.modifyABCDVar(&status, ST_SEND_MASK, 0);       // Route mask off
	if (status.returnCode != 0)
	    throw SctApiException(status.returnCode, "TimeWalkTest::startTest", __FILE__, __LINE__);
	
	name = "TimeWalkTest";
	testVariable = ST_QCAL;                     // QCAL
	
	charges.push_back(1.25);
	charges.push_back(1.50);
	charges.push_back(1.75);
	charges.push_back(2.00);
	charges.push_back(3.0);
	charges.push_back(4.00);
	charges.push_back(5.00);
	charges.push_back(6.00);
	charges.push_back(8.00);
	charges.push_back(10.00);

	testPoints = charges;
	nScans = testPoints.size();
    }
    
    virtual Sct_SctApi_T_Scan* getNextScan(unsigned long index) {
	if (api == 0) 
	    throw InvariantViolatedError("Api pointer is 0 in getNextScan!", __FILE__, __LINE__);	
	
	Sct_SctApiStatus status;
	api->modifyABCDVar(&status, ST_QCAL, charges[index]);
	if (status.returnCode != 0)
	    throw SctApiException(status.returnCode, "TimeWalkTest::getNextScan", __FILE__, __LINE__);
	
	ScanLibrary& sl = CalibrationControllerImpl::instance().getScanLibrary();
        return &sl.strobeDelayScan(1000);
    }
    
    virtual bool canFeedback() {
	return false;
    }
    
    virtual void endTest() {
	charges.clear();
	api = 0;
    }

private:
    TimeWalkTest() {}
    Sct_SctApi_T_SctApi* api;
    std::vector<double> charges;
};
}
}
#endif //TIMEWALKTEST_H
