/************************************************************************************
 *
 * comRegDfns.h:  Defines bits in the registers of the RRIF dual port register block
 *               and the master-slave communication mailbox (in each slave IDRAM).
 *
 *  Damon Fasching, UW Madison (510)486-5230               fasching@wisconsin.cern.ch
 *
 *  modifications/bugs    
 *   - Reserved bits in the command register for slave IDs            05.05.02 dpsf
 ************************************************************************************/
#ifndef COM_REG_DFNS
#define COM_REG_DFNS

#include "smSendTxtBuff.h"         /* pick up N_TXT_BUFFS */
// #include "rodConfiguration.h"      /* pick up N_SLV_DSPS */
#define N_SLV_DSPS 4

/* STATUS_REG_0 */
#define SR_RUNNING                0
#define SR_BUSY                   (1 + (SR_RUNNING))
#define SR_EXECUTING              (1 + (SR_BUSY))
#define SR_PAUSED                 (1 + (SR_EXECUTING))

#define SR_OUT_LIST_RDY           (1 + (SR_PAUSED))
#define SR_DSP_ACK                (1 + (SR_OUT_LIST_RDY))

#define SR_DMA_ACCESS_ACK         (1 + (SR_DSP_ACK))
#define SR_DMA_ACCESS_ERR         (1 + (SR_DMA_ACCESS_ACK))

#define SR_TXT_BUFF_NE(x)         (1 + (x) + (SR_DMA_ACCESS_ERR))
/* the next available register bit is (1 + (N_TXT_BUFFS) + (SR_DMA_ACCESS_ERR)) */

#define SR_TXT_BUFF_PROC(x)       (1 + (x) + (SR_TXT_BUFF_NE(N_TXT_BUFFS-1)))

#define SR_NTASKS                 (1 + (SR_TXT_BUFF_PROC(N_TXT_BUFFS-1)))
#define SR_NTASKS_W               3
#define SR_EVM                    (3 + (SR_NTASKS))          /* Slave DSPs */
#define SR_IDLS_ACTIVE            (1 + (SR_EVM))             
#define SR_IDLP_ACTIVE            (1 + (SR_IDLS_ACTIVE))     /* Master DSP */

/* STATUS_REG_1 */
#define SR_PRIM_LIST_INDEX      0
#define SR_PRIM_LIST_INDEX_W    4
#define SR_PRIM_INDEX          ((SR_PRIM_LIST_INDEX) + (SR_PRIM_LIST_INDEX_W))
#define SR_PRIM_INDEX_W        20

/* STATUS_REG_2 */
#define SR_SLVCOMM(x)             (x)
#define SR_HLTS(x)                (1 + (x) +(SR_SLVCOMM(N_SLV_DSPS-1)))


/* COMMAND_REG_0 */
#define CR_IN_LIST_RDY            0
#define CR_PAUSE                  (1 + (CR_IN_LIST_RDY))
#define CR_RESUME                 (1 + (CR_PAUSE))
#define CR_ABORT                  (1 + (CR_RESUME))
#define CR_TXT_BUFF_RR(x)         (1 + (x) + (CR_ABORT))
/* the next available register bit is (1 + (N_TXT_BUFFS) + (CR_ABORT))  */
#define CR_ID                     (1 + CR_TXT_BUFF_RR(N_TXT_BUFFS-1))
#define CR_ID_W                   2   /* IDs for slave DSPs- on master, unused. */
#define CR_DMA_ACCESS_REQ         ((CR_ID_W) + (CR_ID))
#define CR_FLUSH_EVENTS           (1 + (CR_DMA_ACCESS_REQ))   /* Slave DSPs */
#define CR_IDLP_ACTIVE            (1 + (CR_FLUSH_EVENTS))     /* Slave DSPs */

/* Expert Register */
#define ER_DISP_ROD_REG_ID       0
#define ER_DISP_SERIAL_LOOPS     1
#define ER_DISP_LINK_DATA        2
#define ER_HISTO_1               3
#define ER_DISP_MIRROR_CTIME     4
#define ER_HCTRL_CAPTURE_SET     5
#define ER_STEP_CTRL             6
#define ER_STEP_TRIGGER          7
#define ER_DELAY                 8
#define ER_SOFT_BC_RESET         9
#define ER_MUTE                  10
#define ER_ARDY                  11
#define ER_MAINLOOP_TOGGLE       12
#define ER_ROUTINE_ENTER         13
#define ER_MAINLOOP_DELAYLOOPS   14
#define ER_TIMING_DISPLAY        15
#define ER_DMA_ACK               16
#define ER_SLAVE_WRITE_TEST      17
#define ER_ISR_DELAY             18


/* Histogram Command & Status Registers */
#if    (defined(I_AM_LINUX_HOST)||defined(I_AM_MASTER_DSP))
	#define HCMD_BINEVT     0
	#define HCMD_BINEVT_W   12
	#define HCMD_SET        12
	#define HCMD_SET_W      4
	#define HCMD_BIN        16
	#define HCMD_BIN_W      16

	#define HSR0_SLV0        0
	#define HSR0_SLV1        8
	#define HSR0_SLV2        16
	#define HSR0_SLV3        24
	#define HSR0_SLV_W       8
	
#endif

/* Definitions of the slave's command and status registers are different; 
   master must know both. */
#define HCMD_SLV_BINEVTS    0
#define HCMD_SLV_BINEVTS_W  12
#define HCMD_SLV_SET        12
#define HCMD_SLV_SET_W      4
#define HCMD_SLV_BIN        16
#define HCMD_SLV_BIN_W      16

#define HSR0_SLV_RDY    0
#define HSR0_SLV_EXP    1
#define HSR0_SLV_PROC   2
#define HSR0_SLV_DONE   3

#define HSR0_SLV_EVTS_REC    8
#define HSR0_SLV_EVTS_REC_W  12
#define HSR0_SLV_SET         20
#define HSR0_SLV_SET_W       4
#define HSR0_SLV_BIN         24
#define HSR0_SLV_BIN_W       8

#define HSR1_SLV_EVT_LEN       0
#define HSR1_SLV_EVT_LEN_W    16
#define HSR1_SLV_EVT_PTIME    16
#define HSR1_SLV_EVT_PTIME_W  16


/* Trap status register 0 */
#define TSR0_TRANSMIT          0
#define TSR0_TRAILER           1
#define TSR0_HEADER            2

#define TSR0_ERROR             4
#define TSR0_HEADER_ERROR      5
#define TSR0_TRAILER_ERROR     6
#define TSR0_DATA_ERROR        7

#define TSR0_EVT_FRAME_CNT     8
#define TSR0_EVT_FRAME_CNT_W   8

#define TSR0_IFRAME_CNT        16
#define TSR0_IFRAME_CNT_W      8
#define TSR0_XFRAME_CNT        24
#define TSR0_XFRAME_CNT_W      8

/* Trap status register 1 */
#define TSR1_EVT_TOTAL_PROC_TIME       0
#define TSR1_EVT_TOTAL_PROC_TIME_W     16
#define TSR1_IFRAME_RELEASE_CNT        16
#define TSR1_IFRAME_RELEASE_CNT_W      8
#define TSR1_XFRAME_RELEASE_CNT        24
#define TSR1_XFRAME_RELEASE_CNT_W      8



/* The loop register (increments once per loop */
#define LR_CNT                0
#define LR_CNT_W             32


/* INTR_DSP_HSHK_WR and INTR_DSP_HSHK_RD */
/* The following bits are defined in each of these registers.  They are used for the
 * handshake between the master and a slave when either sends a list to the other.
 * The state machine is simpler than for host-originated lists, e.g. there is no
 * option to pause. */
 
#define INTR_DSP_IN_LIST_RDY    (0)
#define INTR_DSP_ABORT          ((INTR_DSP_IN_LIST_RDY) + 1)
#define INTR_DSP_ACK            ((INTR_DSP_ABORT) + 1)
#define INTR_DSP_OUT_LIST_RDY   ((INTR_DSP_ACK) + 1)

#endif

