/************************************************************************************
 * utilities.h
 *
 *   synopsis: Function prototypes and #defines for general purpose utility routines
 *            and macros.
 *
 *  Damon Fasching, UW Madison (510)486-5230               fasching@wisconsin.cern.ch
 ************************************************************************************/

#ifndef UTILITIES
#define UTILITIES

#include "processor.h"

/* register access width (data bus width of EMIF) */

#define BYTE_W   8
#define HWORD_W 16
#define WORD_W  32

/* default value for primitive parameters, function arguments, etc */

#define DEFAULT (0xFFFFFFFF)

/* z is returned as lesser of x or y */

#define MINI(x,y,z)   z=((x)<(y))?(x):(y);

/*
 * macros to set, reset and read fields of variables and registers
 */

#define N_BIT_MASK(wid)           (~(0xFFFFFFFF<<(wid)))
#define FIELD_MASK(bit,wid)       (N_BIT_MASK(wid) << (bit))
#define FIELD_OFF(bit,wid)        (~(FIELD_MASK(bit,wid)))
#define FIELD_VAL(bit,wid,val)    ((val&(N_BIT_MASK(wid))) << bit)

#define SET_VBIT(var,bit)        (var |= (1 << bit))
#define RST_VBIT(var,bit)        (var &= ~(1 << bit))
#define ASGN_VBIT(var,bit,val)   (var = ((RST_VBIT(var,bit)) | (val<<bit)))
#define GET_VBIT(var,bit)        ((var >> bit) & 1)
#define RST_VFLD(var,bit,wid)    (var &= (FIELD_OFF(bit,wid)))
#define ASGN_VFLD(var,bit,wid,val)  \
             (var = (RST_VFLD(var,bit,wid)) | (FIELD_VAL(bit,wid,val)))
#define GET_VFLD(var,bit,wid)    ((FIELD_MASK(bit,wid) & var) >> bit)

/*  Some of the macros below are essentially the same as those in the Texas
 * Instruments provided regs.h.  The ones in this file should be used in DSP code
 * authored by the ROD group.  If Texas Instruments changes their macro names for
 * some reason, the DSP code will not be affected.                  dpf             */
 
#define CONTENTS(adr) (*((volatile UINT32 *)(adr)))

#define READ_REG(adr)             (CONTENTS(adr))
#define WRITE_REG(adr,val)        (CONTENTS(adr) = (val))
#define SET_RBIT(adr,bit)         (CONTENTS(adr) |= (1 << (bit)))
#define RST_RBIT(adr,bit)         (CONTENTS(adr) &= (~(1 << (bit))))
#define ASGN_RBIT(adr,bit,val)    (CONTENTS(adr) = (RST_RBIT(adr,bit)) | (val<<bit))
#define GET_RBIT(adr,bit)         ((CONTENTS(adr) >> bit) & 1)
#define RST_RFLD(adr,bit,wid)     (CONTENTS(adr) &= (FIELD_OFF(bit,wid)))
#define GET_RFLD(adr,bit,wid)     ((CONTENTS(adr) & FIELD_MASK(bit,wid)) >> bit)
#define ASGN_RFLD(adr,bit,wid,val) (CONTENTS(adr) = \
                                   (RST_RFLD(adr,bit,wid)) | (FIELD_VAL(bit,wid,val)))

#endif
