#ifndef DEFAULTVARIABLE_H
#define DEFAULTVARIABLE_H

#include "ConfigurationVariable.h"

namespace SctData {
    
/**
  This class represents a configuration variable for which there is no specific class.
  The actual<->logical mapping is therefore actual=logical.
  It can also be used as an implementation class for variables that have that same mapping.
  @todo Think: Is this necessary?  Does it imply public virtual inheritance of
  ConfigurationVariable ...are there any problems?
  */
class DefaultVariable : public ConfigurationVariable {
public:
    DefaultVariable(unsigned int typeRep);
    
    //Conversions
    virtual double getLinkActualPoint(double logicalPt, const ModuleConfiguration& config, unsigned int link) const throw();    
    virtual double getChipActualPoint(double logicalPt, const ModuleConfiguration& config, unsigned int chip) const throw();
    virtual double getChannelActualPoint(double logicalPt, const ModuleConfiguration& config, unsigned int channel) const throw();    
    
    virtual double getLinkLogicalPoint(double actualPt, const ModuleConfiguration& config, unsigned int link) const throw();
    virtual double getChipLogicalPoint(double actualPt, const ModuleConfiguration& config, unsigned int chip) const throw();    
    virtual double getChannelLogicalPoint(double actualPt, const ModuleConfiguration& config, unsigned int channel) const throw();    
    
    //ModuleConfiguration interaction
    /**
      Obviously, these can't work for a DefaultVariable.
      */
    virtual double getChipActualPoint(const ModuleConfiguration& config, unsigned int chip) const throw();
    virtual void setChipActualPoint(double actualPt, ModuleConfiguration& config, unsigned int chip) const throw();

    
protected:
    //DefaultVariable(unsigned int typeVal, string variableName, string strategyDescription);
};
}

#endif //#ifndef DEFAULTVARIABLE_H
