#include "ErfFitObject.h"

using namespace Sct;

namespace SctData {
ErfFitObject::ErfFitObject() throw() {
    this->init();
    this->reset();
}

ErfFitObject::ErfFitObject(const TF1& f) throw(LogicError) {
    this->init();
    this->reset();
    *this=f;
}

auto_ptr<FitObject> ErfFitObject::clone() const throw() {
    return auto_ptr<FitObject>(new ErfFitObject(*this));
}

ErfFitObject& ErfFitObject::operator= (const TF1& f) throw(LogicError) {
    this->FitObject::operator=(f);
    return *this;
}

void ErfFitObject::init() throw() {
    this->setNDim(1);
    this->setNPar(3);
    try {
	this->setParName(0,"Prefact");
	this->setParName(1,"Mean");
	this->setParName(2,"Sigma");
    } catch (LogicError&) {} // Throw away LogicError that should never happen
    this->setFormula("erf");
}

// Call the appropriate constructor for making a ROOT TF1 object from this ErfFitObject::
auto_ptr<TF1> ErfFitObject::makeBasicRootTF1() const throw(LogicError) {
    auto_ptr<TF1> r( new ::TF1("myerf", erf, this->getVarMin(0), this->getVarMax(0), 3));
    if (!r.get())
        throw InvariantViolatedError("ErfFitObject::makeBasicRootTF1()", __FILE__, __LINE__);
    return r;
}

}  // end of namespace SctData

