#include "GausFitObject.h"
#include "TF1.h"
#include "Sct/LogicErrors.h"

// Over-ride function...
// Calls base-class equality operator then renames the parameters

namespace SctData {

GausFitObject::GausFitObject() throw() {
    this->init();
    this->reset();
}

GausFitObject::GausFitObject(const TF1& f) throw(LogicError) {
    this->init();
    this->reset();
    *this=f;
}

auto_ptr<FitObject> GausFitObject::clone() const throw() {
    return auto_ptr<FitObject> (new GausFitObject(*this));
}

GausFitObject& GausFitObject::operator= (const TF1& rootfunc) throw(LogicError) {
    //      if (!strcmp(rootfunc.GetName(), "gaus")) {
    //	  ostringstream os; os <<"GausFitObject::operator=(TF1&) not a gaus TF1: "
    //			       <<rootfunc.GetName();
    //	  throw InvalidArgument(os.str());
    //      }
    this->FitObject::operator=(rootfunc);
    return (*this);
}

auto_ptr<TF1> GausFitObject::makeBasicRootTF1() const throw(LogicError) {
    auto_ptr<TF1> r ( new TF1("mygaus","gaus"));
    if (!r.get())
        throw InvariantViolatedError("GausFitObject::makeBasicRootTF1()", __FILE__, __LINE__);
    return r;
}

void GausFitObject::init() throw() {
    this->setNDim(1);
    this->setNPar(3);
    //Throw away LogicError that should never happen
    try {
	this->setParName(0,"Prefact");
	this->setParName(1,"Mean");
	this->setParName(2,"Sigma");
    } catch (LogicError&) {}
    this->setFormula("gaus");
}

} // end of namespace SctData
