#include "../NoiseOccupancyTestResult.h"
#include "../RootStreamableAdapter.h"
#include "NoiseOccupancyTestResultStreamer.h"

#include <TGraph.h>
#include <TF1.h>

using namespace Sct;

namespace SctData {
namespace IO {


NoiseOccupancyTestResultStreamer::NoiseOccupancyTestResultStreamer() throw() {}

bool NoiseOccupancyTestResultStreamer::inMap = IOManager::addToMap("SctData::NoiseOccupancyTestResult", auto_ptr<Streamer>(new NoiseOccupancyTestResultStreamer()));

shared_ptr<Streamable> NoiseOccupancyTestResultStreamer::read(IStream& in, const IOManager& manager) const throw(LogicError, IoError) {
    shared_ptr<Streamable> ad (new NoiseOccupancyTestResult());
    read(in, *ad, manager);
    return ad;
}

void NoiseOccupancyTestResultStreamer::write(OStream& out, const Streamable& ob, const IOManager& manager) const throw(LogicError, IoError) {
    //Let superclass do its work
    TestResultStreamer::write(out, ob, manager);

    const NoiseOccupancyTestResult& mytest = dynamic_cast<const NoiseOccupancyTestResult&>(ob);
    for (unsigned int ichip=0; ichip<nChipModule; ichip++) {
        writeChipResult(out, mytest.getChipResult(ichip), manager);
    }
}

void NoiseOccupancyTestResultStreamer::read(IStream& in, Streamable& ob, const IOManager& manager) const throw(LogicError, IoError) {
    TestResultStreamer::read(in, ob, manager);

    NoiseOccupancyTestResult& mytest = dynamic_cast<NoiseOccupancyTestResult&>(ob);
    for (unsigned int ichip=0; ichip<nChipModule; ichip++) {
        readChipResult(in, mytest.getChipResult(ichip), manager);
    }
}

void NoiseOccupancyTestResultStreamer::writeChipResult(OStream& out, const ChipNOResult& c, const IOManager& manager) const throw (LogicError, IoError) {
    manager.writeImpl(out, RootStreamableAdapter( *c.graph ), true);
    manager.writeImpl(out, RootStreamableAdapter( *c.fit ), false);
    out << c.mean << c.rms << c.offset;
}

void NoiseOccupancyTestResultStreamer::readChipResult(IStream& in, ChipNOResult& c, const IOManager& manager) const throw(LogicError, IoError) {
    shared_ptr<Streamable> ob = manager.readImpl(in);
    shared_ptr<RootStreamableAdapter> ad  = boost::dynamic_pointer_cast<RootStreamableAdapter>(ob);
    c.graph = shared_ptr<TGraph> ( dynamic_cast<TGraph*>(ad->getObject()));
    
    auto_ptr<TF1> f = NoiseOccupancyTestResult::createFitFunction();
    RootStreamableAdapter ad2( *f.release() );
    manager.readImpl(in, ad2, false);
    c.fit = shared_ptr<TF1> ( dynamic_cast<TF1*>(ad2.getObject()) );
    in >> c.mean >> c.rms >> c.offset;
}

}
}
