#include "TestResultStreamer.h"
#include "../TestResult.h"
#include "../ConfigurationVariable.h"

#include <string>

using namespace std;
using namespace Sct;

namespace SctData {
namespace IO {

TestResultStreamer::TestResultStreamer() throw() {}

void TestResultStreamer::write(OStream& out, const Streamable& ob, const IOManager& manager) const throw(LogicError, IoError) {
    // cout <<"TestResultIS publishing"<<endl;
    const TestResult& mytest = dynamic_cast<const TestResult&>(ob);
    
    out << mytest.getRunNumber() << mytest.getModuleName(); /// This is because ISostream operator<<(string&) isnt const

    manager.writeImpl(out, mytest.getTestVariable(), false);
    manager.writeImpl(out, mytest.getScanVariable(), false);

    out << mytest.getNScans();
    for (unsigned int i=0; i<mytest.getNScans(); ++i) {
        out << mytest.getScanNumberAt(i) << mytest.getTestPointAt(i);
    }
    manager.writeImpl(out, mytest.getDefects() , false);
    manager.writeImpl(out, mytest.getUpdatedConfiguration(), false);
    
    out << mytest.getPassed();
    out << mytest.getProblem();
    vector<string> comments = mytest.getComments();
    out << comments.size();
    for (unsigned int i=0; i<comments.size(); ++i) {
	out << comments[i];
    }
}

void TestResultStreamer::read(IStream& in, Streamable& ob, const IOManager& manager) const throw(LogicError, IoError) {  
    TestResult& mytest = dynamic_cast<TestResult&>(ob);

    //cout << "TestResultIS refreshing" << endl;

    unsigned int runNumber = 0;
    string moduleName;

    in >> runNumber >> moduleName;

    shared_ptr<Streamable> ob2 = manager.readImpl(in, "SctData::ConfigurationVariable");
    mytest.setTestVariable(*boost::dynamic_pointer_cast<ConfigurationVariable>(ob2));	///Should always succeed!
    
    ob2 = manager.readImpl(in, "SctData::ConfigurationVariable");
    mytest.setScanVariable(*boost::dynamic_pointer_cast<ConfigurationVariable>(ob2));	///Should always succeed!
    
    mytest.setRunNumber(runNumber);
    mytest.setModuleName(moduleName);
    
    //cout << "   found "<<runNumber << moduleName << endl;

    unsigned int size=0;
    in >> size;
    for (unsigned int i=0; i<size; ++i) {
        unsigned int scanNumber;
        double testPoint;
        in >> scanNumber >> testPoint;
        mytest.addScan(scanNumber, testPoint);
        //cout << "TestResultIS refresh : added scan "<<scanNumber<<endl;
    }
    manager.readImpl(in, mytest.getDefects(), false);
    manager.readImpl(in, mytest.getUpdatedConfiguration(), false);
    
    bool passed, problem;
    unsigned int nComments;    
    string comment;
    in >> passed;   mytest.setPassed(passed);
    in >> problem;  mytest.setProblem(problem);
    
    in >> nComments;
    for (unsigned int i=0; i<nComments; ++i) {
	in >> comment;	mytest.addComment(comment);
    }
    
    //cout << "TestResultIS refresh done" << endl;
}
}
}

