#include "Sct/SctParameters.h"
#include "OccupancyProjector.h"
#include "ModuleDefectList.h"
#include "TH1.h"
#include "TH2.h"

namespace SctData{
    using namespace Sct;

    OccupancyProjector::OccupancyProjector() throw() : raw(0) {
	unsetVetos();
    }

    OccupancyProjector::OccupancyProjector(const RawScanResult& r) throw() : raw(&r) {
	unsetVetos();
    }

    OccupancyProjector::~OccupancyProjector() throw() {;}
    
    void OccupancyProjector::vetoAllDefects(const ModuleDefectList& l) throw(){
	defects=&l; m_vetoAllDefects=true;
    }
    void OccupancyProjector::vetoSevereDefects(const ModuleDefectList& l) throw(){
	defects=&l;
    }
    //void OccupancyProjector::setRawScanResult(const RawScanResult& r) throw(){
    //	raw=&r; unsetVetos();
    //}
    void OccupancyProjector::vetoMaskedChannels(const ModuleConfiguration& c) throw(){
	config=&c;
    }
    void OccupancyProjector::unsetVetos() throw(){
	config=0; defects=0; m_vetoAllDefects = false;
    }

    auto_ptr<TH1> OccupancyProjector::getOccupancy(const char* name, const ModuleElement& element) const throw (LogicError) {
	if (!raw) throw IllegalStateError("OccupancyProjector::getOccupancy no raw result", __FILE__, __LINE__);

	double* bins = raw->getPoints().getEdges();
	auto_ptr<TH1> occ ( new TH1D(name, "Raw data projection", raw->getPoints().getNPoints(), bins) );
	delete[] bins;
	
	//Explicit binomial errors.
	for (unsigned int j=0; j<raw->getPoints().getNPoints(); ++j){
	    Stat_t n=raw->getPoints().getNEvents(j); 
	    double z=0;
	    unsigned int nChans = 0;
	    
	    //Do projection, taking into account that we might want data on both links
	    for (unsigned int i=element.getFirst(); i<=element.getLast(); ++i) {
		if (config) {
		    if (config->channelIsMasked(i) ) continue;
		}
		if (defects) {
		    if (m_vetoAllDefects && defects->defectAffectingElement(Channel(i)) ) continue;
		    if (defects->severeDefectAffectingElement(Channel(i)) ) continue;
		}
		// Make sure to get correct scan data TH2:
		z += raw->getScanData(i/nChannelLink).GetBinContent(i%nChannelLink+1, j+1);
		nChans++;
	    }
	    
	    if (n==0 || nChans==0) {
		// no triggers.
		occ->SetBinContent(j+1, z);
		occ->SetBinError(j+1, 0);
		continue;
	    }
	    n*=nChans;                         // number of triggers corrected for #channels
	    
	    double err2=(z+1)*(n-z+1)/(n*n*n);
	    double occupancy=z/n;
	    
	    occ->SetBinContent(j+1, occupancy);
	    occ->SetBinError(j+1,sqrt(err2));
	}       
	return occ;
    }
    
}// end of namepace SctData
