#ifndef SCANRESULT_H
#define SCANRESULT_H

#include "Sct/Serializable.h"
#include "Sct/Exception.h"
#include "ScanHeader.h"
#include "ModuleConfiguration.h"
#include "ScanPoints.h"

namespace SctData {
		
/**
This class represents all possible things what might come out of IS.
@author Matthew Palmer
@date 28/02/2003
*/
class ScanResult : public virtual Sct::Serializable {
public:
    /**
       Creates a ScanResult by copying the header and module configuration
    */
    ScanResult(const ScanHeader& header, const ModuleConfiguration& moduleConfig, const ScanPoints& points) throw();
    
    /**
       Copy constructor
    */
    ScanResult(const ScanResult& copyMe) throw();
    
    virtual ~ScanResult() throw() ;
    
    /// Retrieves the scan-header object
    const ScanHeader& getHeader() const throw() {return header;}
    
    /// Returns the module configuration
    const ModuleConfiguration& getConfiguration() const throw() {return moduleConfig;}
    
    
    /// Get the ScanPoints object
    const ScanPoints& getPoints() const throw() {return points;}
    
protected:
    const ScanHeader header;					///< The ScanHeader that identifies this ScanResult.
    const ModuleConfiguration moduleConfig;		///< The ModuleConfiguration of the module referred to in the header.
    const ScanPoints points;					///< The scan points.
    
//Builder stuff
protected:
    friend class ScanResultIOHelper;
    ScanResult() throw() ;								///< For IO use only.
};
	

}
#endif //#ifndef SCANRESULT_H
