#include "TopHatFitObject.h"
#include "TF1.h"
#include "Sct/LogicErrors.h"

using namespace Sct;

namespace SctData {

TopHatFitObject::TopHatFitObject() throw() {
    this->init();
    this->reset();
}

TopHatFitObject::TopHatFitObject(const TF1& f) throw(LogicError) {
    this->init();
    this->reset();
    *this=f;
}

auto_ptr<FitObject> TopHatFitObject::clone() const throw() {
    return auto_ptr<FitObject>(new TopHatFitObject(*this));
}

TopHatFitObject& TopHatFitObject::operator=(const TF1& f) throw(LogicError) {
    this->FitObject::operator=(f);
    return *this;
}

void TopHatFitObject::init() throw() {
    this->setFormula("tophat");
    this->setNDim(1);
    this->setNPar(5);
    try {
	this->setParName(0,"Prefact");
	this->setParName(1,"MidRise");
	this->setParName(2,"RiseTime");
	this->setParName(3,"MidFall");
	this->setParName(4,"FallTime");
    } catch (LogicError&) {} //Throw away LogicError that should never happen
}
// Call the appropriate constructor for making a ROOT TF1 object from this
// TopHatFitObject::
auto_ptr<TF1> TopHatFitObject::makeBasicRootTF1() const throw(LogicError) {
    auto_ptr<TF1> r (new ::TF1("tophat", tophat, this->getVarMin(0), this->getVarMax(0), 5));
    if (!r.get())
        throw IllegalStateError("TopHatFitObject::makeBasicRootTF1()", __FILE__, __LINE__);
    return r;
}

} // end of namespace SctData
