#include "SctData/NPtGainTestResult.h"
#include "SctData/ModuleDefect.h"
#include "SctData/FitScanResult.h"
#include "SctData/ResponseCurve.h"
#include "Sct/IS/IOManagerIS.h"
#include "Sct/IS/IONameIS.h"
#include "Sct/SctNames.h"
#include "Sct/SctParameters.h"
#include "Sct/Exception.h"

#include <is/isinfo.h>
#include <TGraph.h>
#include <TCanvas.h>
#include <TF1.h>
#include <TSystem.h>
#include <TApplication.h>

#include <iomanip>
#include <algorithm>

using namespace Sct;
using namespace SctData;
using namespace std;
using namespace Sct::IS;
using namespace boost;

int Help() {
    cout << "This program shows the contents of NPtGain TestResults.  It takes one argument - the name of the NPtGainTestResult in IS." << endl;
    return 0;
}

TCanvas* createCanvas(string name, string title) {
    return new TCanvas(name.c_str(), title.c_str());
}

//This program assumes all FitScanResults in IS are part of an NptGain Test.
//Also assumes sequence for test points.
int main(int argc, char **argv) {
    setExceptionHandlers(argv[0]);

    if (argc == 1)
        return Help();
    
    //Create TApplication - necessary to prevent batch mode and no graphics on later ROOT versions
    TApplication myapp("myapp", 0, 0);

    shared_ptr<NPtGainTestResult> result;

    try{
	
	cout << "Get NPtGainTestResult" << endl;
	shared_ptr<Serializable> ob = IOManagerIS::instance().read(string(argv[1]));
	cout << "Done: "<< ob << endl;
	
	result = dynamic_pointer_cast <NPtGainTestResult>(ob);
	if (!result) {
	    cerr << "Errgh, didn't get a NPtGainTestResult - who knows why??? " << ob << endl;
	    cout << ob->getClassName() << endl;
	    return -1;
	}

    }catch (Sct::Throwable& e){
	cout << e.what() << endl;
	e.sendToMrs(MRS_FATAL);
	return -1;
    }

    //Print out chip fit parameters
    cout << "Chip parameters - suitable for copying and pasting into TestNO" << endl;
    for (unsigned int i = 0; i < 12; ++i) {
        shared_ptr<TF1> f = result->getChipData(i).rc->getFunction();
        cout << f->GetParameter(0) << "  " << f->GetParameter(1) << "  " << f->GetParameter(2) << endl;
    }

    //Now print out defects:
    cout << endl << "Defects: " << endl;
    result->getDefects().print(cout) << endl;
    
    //Construct gain, noise and offset graphs
    TGraph* gain = new TGraph(nChannelModule);
    TGraph* noise = new TGraph(nChannelModule);
    TGraph* offset = new TGraph(nChannelModule);
    for (unsigned int i=0; i<nChannelModule; ++i) {
        const NPtGainTestResultData& data = result->getChannelData(i);
        gain->SetPoint(i, i, data.gain);
        noise->SetPoint(i, i, data.noise);
        offset->SetPoint(i, i, data.offset);
    }

    createCanvas(result->getModuleName() + "_gain", result->getModuleName() + " Gain");
    gain->SetTitle("Gain");
    gain->Draw("alp");
    createCanvas(result->getModuleName() + "_noise", result->getModuleName() + " Noise");
    noise->SetTitle("Noise");
    noise->Draw("alp");
    createCanvas(result->getModuleName() + "_offset", result->getModuleName() + " Offset");
    offset->SetTitle("Offset");
    offset->Draw("alp");

    TCanvas *c = createCanvas(result->getModuleName() + "_chip", result->getModuleName() + " Chip Fits");
    c->Divide(4, 3);
    for (unsigned int i = 0; i < nChipModule; ++i) {
        c->cd(i + 1);
        TGraph & g = *result->getChipData(i).graph;
        g.SetMarkerStyle(20);
        g.Draw("ap");
        shared_ptr<TF1> f = result->getChipData(i).rc->getFunction();
        f->SetRange(0, 8);
	f->SetFillStyle(0);
        f->Draw("same");
    }

    myapp.Run();
}
