#include "SctData/RawScanResult.h"
#include "SctData/FitScanResult.h"
#include "SctData/ModuleDefect.h"
#include "SctData/FitObject.h"
#include "Sct/IS/IOManagerIS.h"
#include "SctData/OccupancyProjector.h"
#include "SctData/ModuleElement.h"

#include <TH1.h>
#include <TF1.h>
#include <TCanvas.h>
#include <TApplication.h>

#include <cstdio>
#include <iomanip>
#include <boost/shared_ptr.hpp>

using namespace std;
using namespace SctData;
using namespace Sct::IS;
using namespace boost;

//Basic idea is take args:
//FitScanResult name
//Start channel
//End channel
int main(int argc, char** argv) {
    setExceptionHandlers(argv[0]);
    //Create TApplication - necessary to prevent batch mode and no graphics on later ROOT versions
    TApplication myapp("myapp", 0, 0);

    if (argc < 4)
        return -1;
    char* name = argv[1];
    int channelStart = atoi(argv[2]);
    int channelEnd = atoi(argv[3]);

    shared_ptr<Serializable> ob = IOManagerIS::instance().read(string(name));
    shared_ptr<FitScanResult> scan = dynamic_pointer_cast<FitScanResult>(ob);

    if (scan == 0)
        return -2;

    cout << "Got FitScanResult now get RawScanResult" << endl;

    string rawName = RawScanResult::getUniqueID(scan->getHeader());
    cout << rawName << endl;
    ob = IOManagerIS::instance().read("EventData.SctData::RawScanResult."+ rawName);
    shared_ptr<RawScanResult> raw = dynamic_pointer_cast<RawScanResult>(ob);

    if (raw == 0)
        return -3;

    int j=1;
    TCanvas* c = new TCanvas();
    c->Divide(4,3);
    
    OccupancyProjector occ(*raw);
    vector<shared_ptr<TH1> > hists;
    vector<shared_ptr<TF1> > funcs;

    for (unsigned int i=0; i<scan->getNChipFits(); ++i) {
        c->cd(i+1);
        char histName[100];
        sprintf(histName, "chipHist%i", i);
        auto_ptr<TH1> h = occ.getOccupancy(histName, Chip(i));
        h->Draw();
        FitObject& fo = scan->getChipFit(i);
        auto_ptr<TF1> f = fo.makeRootTF1();
        //f->SetMaximum(1);
	f->SetLineColor(2);
	f->SetLineWidth(1);
	f->SetFillStyle(0);
        f->Draw("same");
	
	hists.push_back(shared_ptr<TH1>(h));
	funcs.push_back(shared_ptr<TF1>(f));
	fo.print();
    }

    if ((unsigned)channelEnd > scan->getNChannelFits()) channelEnd = scan->getNChannelFits();
    if (channelEnd > channelStart) {
	c = new TCanvas();
	c->Divide(6,6);	
    }
    
    for (int i=channelStart; i<channelEnd; ++i) {
        if (j==37) {
            c = new TCanvas();
            c->Divide(6,6);
            j=1;
        }
        c->cd(j);

        char histName[100];
        sprintf(histName, "channelHist%i", i);
        auto_ptr<TH1> h = occ.getOccupancy(histName, Channel(i) );
        h->Draw();

        FitObject& fo = scan->getChannelFit(i);
        auto_ptr<TF1> f ( fo.makeRootTF1() );
	f->SetLineColor(2);
	f->SetLineWidth(1);
        //f->SetMaximum(1);
	f->SetFillStyle(0);
        f->Draw("same");
        ++j;
	
	hists.push_back(shared_ptr<TH1>(h));
	funcs.push_back(shared_ptr<TF1>(f));
    }

    //Now print out defects:
    cout << endl << "Defects: " << endl;
    scan->getDefects().print(cout) << endl;
    
    myapp.Run();

    return 0;
}
