#include "SctData/TrimRangeTestResult.h"
#include "SctData/ModuleDefect.h"
#include "SctData/FitScanResult.h"
#include "Sct/IS/IOManagerIS.h"
#include "Sct/IS/IONameIS.h"
#include "Sct/SctNames.h"
#include "Sct/SctParameters.h"
#include "Sct/Exception.h"

#include <is/isinfo.h>
#include <TGraph.h>
#include <TCanvas.h>
#include <TH2.h>
#include <TF1.h>
#include <TLatex.h>
#include <TApplication.h>

#include <iomanip>
#include <algorithm>

using namespace Sct;
using namespace SctData;
using namespace std;
using namespace Sct::IS;
using namespace boost;

int Help() {
    cout << "This program shows the contents of TrimRange TestResults.  It takes one argument - the name of the TrimRangeTestResult in IS." << endl;
    return 0;
}

TCanvas* createCanvas(string name, string title) {
    return new TCanvas(name.c_str(), title.c_str(), 900, 700);
}

//Assumes sequence for test points.
int main(int argc, char **argv) {
    setExceptionHandlers(argv[0]);

    if (argc == 1)
        return Help();

    shared_ptr<TrimRangeTestResult> result;

    try{

	string fullname = string(argv[1]);
	cout << "Get " << fullname << endl;
	shared_ptr<Serializable> ob = IOManagerIS::instance().read(fullname);
	
	result = dynamic_pointer_cast <TrimRangeTestResult>(ob);
	if (!result) {
	    cerr << "Errgh, didn't get a TrimRangeTestResult - who knows why??? " << ob << endl;
	    cout << ob->getClassName() << endl;
	    return -1;
	}
    }catch (Sct::Throwable& e){
	e.sendToMrs(MRS_FATAL); cerr << e.what()<< endl; return 1;
    }catch (std::exception& e){
	cerr << e.what() << endl;
    }catch (...){
	cerr << "URRGH! caught I don't know what!"<<endl;
    }


    //Now print out defects:
    cout << endl << "Defects: " << endl;
    result->getDefects().print(cout) << endl;

    //Create TApplication - necessary to prevent batch mode and no graphics on later ROOT versions
    TApplication myapp("myapp", 0, 0);

    TCanvas *c = createCanvas(result->getModuleName() + "_TrimData", result->getModuleName() + " Trim Range Fits");
    c->Divide(4, 3);
    for (unsigned int ichip = 0; ichip < nChipModule; ++ichip) {
        c->cd(ichip + 1);
	char name[100]; sprintf(name,"Chip_%d",ichip);
	TH2D* hist = new TH2D(name,name,10,0,300,10,0,16);
	hist->Draw();

	int range=result->chipTrim[ichip]->range;
	char txt[10];
	sprintf(txt,"R%d",range);
	TLatex* lab=new TLatex();
	lab->SetTextSize(0.1);
	lab->DrawLatex(0,0,txt);

	for (unsigned ichannel=0; ichannel<nChannelChip; ++ichannel){
	    if (result->chipTrimData[ichip]!=0){
		Stat<TrimRangeTestResult::TrimData> trim = result->chipTrimData[ichip]->channelData.getAt(ichannel);
		TGraph* g = new TGraph();
		g->SetMarkerColor(ichannel);
		g->SetMarkerStyle(21);
		g->SetMarkerSize(0.5);
		g->Set(trim.value.graph.size());
		for (unsigned ipt=0; ipt<trim.value.graph.size(); ++ipt){
		    g->SetPoint(ipt,trim.value.graph[ipt].first, trim.value.graph[ipt].second);
		}
		g->SetLineColor(ichannel);
		g->SetLineWidth(1);
		if (g->GetN()!=0 ){
		    g->Draw("LP");
		}
	    }
	}
    }
    c->Print("trimdata.ps");

    TCanvas *c2 = createCanvas(result->getModuleName() + "_Trims", result->getModuleName() + " Trims");
    c2->Divide(1, 4);
    for (unsigned int ilink = 0; ilink < nLinkModule; ++ilink) {
	TGraph* g1 = new TGraph();
	TGraph* g2 = new TGraph();
	g1->SetMarkerColor(2);
	g1->SetMarkerStyle(21);
	g1->SetMarkerSize(1);
	g2->SetMarkerColor(2);
	g2->SetMarkerStyle(21);
	g2->SetMarkerSize(1);

	// find number of trimmed channels:
	int npoints=0;
	for (unsigned ichip=ilink*nChipLink; ichip<(ilink+1)*nChipLink; ++ichip){
	    if (result->chipTrim[ichip]==0){
		cerr << " no trim at chip "<< ichip << endl; continue;
	    }
	    npoints += result->chipTrim[ichip]->channelTrim.n();
	}
	
	g1->Set(npoints);
	g2->Set(npoints);

	int ipoint=0;
	for (unsigned ichannel=ilink*nChannelLink; ichannel<(ilink+1)*nChannelLink; ++ichannel){

	    if (result->chipTrim[ichannel/128]==0){
		continue;
	    }

	    Stat<TrimRangeTestResult::Trim> trim = result->chipTrim[ichannel/128]
		->channelTrim.getAt(ichannel%128);

	    if (trim.valid){
		g1->SetPoint(ipoint,ichannel%nChannelLink,trim.value.trim);
		g2->SetPoint(ipoint,ichannel%nChannelLink,trim.value.vthr);
		++ipoint;
	    }
	}
	
	char name[100]; sprintf(name,"Link_%d_t",ilink);
	TH2D* hist1 = new TH2D(name,name,768,-0.5,767.5,16,-0.5,15.5);
	sprintf(name,"Link_%d_v",ilink);
	TH2D* hist2 = new TH2D(name,name,768,-0.5,767.5,3,0.,300.);
	c2->cd(ilink*2 + 1);
	hist1->Draw();
	g1->Draw("P");
	c2->cd(ilink*2 + 2);
	hist2->Draw("");
	g2->Draw("P");
	for (unsigned ichip=ilink*nChipLink; ichip<(ilink+1)*nChipLink; ++ichip){
	    int range=result->chipTrim[ichip]->range;
	    char txt[10];
	    sprintf(txt,"R%d",range);
	    TLatex* lab=new TLatex();
	    lab->SetTextSize(0.1);
	    lab->DrawLatex(nChannelChip*(ichip%nChipLink)+nChannelChip/2,10,txt);
	}

    }
    
    myapp.Run();
    
}
