#include <iostream>

#include <vector>

#include "SctApiImpl.h"
#include "ABCD/ABCDchip.h"

using namespace SctApi;

TriggerImpl::TriggerImpl() 
{
  incCmd = incData = 0;
}

TriggerImpl::TriggerImpl(const TriggerImpl &other) :
  trigSequence(other.trigSequence), incCmd(other.incCmd), incData(other.incData)
{
}

TriggerImpl::~TriggerImpl() 
{
}

void TriggerImpl::singleL1A() 
{
  trigSequence.clear();

  trigSequence.push_back(std::make_pair(L1_TRIGGER, 0));

  incCmd = incData = 0;
}

void TriggerImpl::calL1A(unsigned short delay) 
{
  trigSequence.clear();

  trigSequence.push_back(std::make_pair(CALIBRATION_PULSE, 0));
  trigSequence.push_back(std::make_pair(DELAY, delay));
  trigSequence.push_back(std::make_pair(L1_TRIGGER, 0));

  incCmd = incData = 0;
}

void TriggerImpl::pulseL1A(unsigned short delay)
{
  trigSequence.clear();

  trigSequence.push_back(std::make_pair(PULSE_INPUT_REG, 0));
  trigSequence.push_back(std::make_pair(DELAY, delay));
  trigSequence.push_back(std::make_pair(L1_TRIGGER, 0));

  incCmd = incData = 0;
}

void TriggerImpl::softL1A(unsigned short delay)
{
  trigSequence.clear();

  trigSequence.push_back(std::make_pair(SOFT_RESET, 0));
  trigSequence.push_back(std::make_pair(DELAY, delay));
  trigSequence.push_back(std::make_pair(L1_TRIGGER, 0));

  incCmd = incData = 0;
}

void TriggerImpl::softCalL1A(unsigned short delay, unsigned short delay2)
{
  trigSequence.clear();

  trigSequence.push_back(std::make_pair(SOFT_RESET, 0));
  trigSequence.push_back(std::make_pair(DELAY, delay));
  trigSequence.push_back(std::make_pair(CALIBRATION_PULSE, 0));
  trigSequence.push_back(std::make_pair(DELAY, delay2));
  trigSequence.push_back(std::make_pair(L1_TRIGGER, 0));

  incCmd = incData = 0;
}
 
void TriggerImpl::softPulseL1A(unsigned short delay, unsigned short delay2)
{
  trigSequence.clear();

  trigSequence.push_back(std::make_pair(SOFT_RESET, 0));
  trigSequence.push_back(std::make_pair(DELAY, delay));
  trigSequence.push_back(std::make_pair(PULSE_INPUT_REG, 0));
  trigSequence.push_back(std::make_pair(DELAY, delay2));
  trigSequence.push_back(std::make_pair(L1_TRIGGER, 0));

  incCmd = incData = 0;
}

void TriggerImpl::bcL1A(unsigned short delay)
{
  trigSequence.clear();

  trigSequence.push_back(std::make_pair(BC_RESET, 0));
  trigSequence.push_back(std::make_pair(DELAY, delay));
  trigSequence.push_back(std::make_pair(L1_TRIGGER, 0));

  incCmd = incData = 0;
}

void TriggerImpl::bcCalL1A(unsigned short delay, unsigned short delay2)
{
  trigSequence.clear();

  trigSequence.push_back(std::make_pair(BC_RESET, 0));
  trigSequence.push_back(std::make_pair(DELAY, delay));
  trigSequence.push_back(std::make_pair(CALIBRATION_PULSE, 0));
  trigSequence.push_back(std::make_pair(DELAY, delay2));
  trigSequence.push_back(std::make_pair(L1_TRIGGER, 0));

  incCmd = incData = 0;
}
 
void TriggerImpl::bcPulseL1A(unsigned short delay, unsigned short delay2)
{
  trigSequence.clear();

  trigSequence.push_back(std::make_pair(BC_RESET, 0));
  trigSequence.push_back(std::make_pair(DELAY, delay));
  trigSequence.push_back(std::make_pair(PULSE_INPUT_REG, 0));
  trigSequence.push_back(std::make_pair(DELAY, delay2));
  trigSequence.push_back(std::make_pair(L1_TRIGGER, 0));

  incCmd = incData = 0;
}

void TriggerImpl::setCommIncr(unsigned short command, unsigned short incr)
{
  incCmd = command;
  incData = incr;
}

void TriggerImpl::getCommIncr(unsigned short &command, unsigned short &incr) const
{
  command = incCmd;
  incr = incData;
}

const ::SctApi::Trigger::RODTriggers TriggerImpl::getRODTriggers() const 
{
  return trigSequence;

//   ::SctApi::Trigger::RODTriggers result;

  

//   return result;
}

void TriggerImpl::copy(const ::SctApi::Trigger &trig) 
{
  short unsigned cmd, data;

  trig.getCommIncr(cmd, data);

  incCmd = cmd;
  incData = data;

  trigSequence = trig.getRODTriggers();
}

ScanDefImpl::ScanDefImpl() 
  : trigsPerBurst(0), 
    scanVariable(0),
    scanVariable2(0),
    trigSequence(new TriggerImpl), trigSequence2(new TriggerImpl),
    format(1),
    full(0),
    bits32(1),
    loopCalLine(0),
    distSlave(0),
    debug(0),
    scanNumber(0),
    runNumber(0)
{
  scanPoints.clear();
  scanPoints2.clear();
}

ScanDefImpl::~ScanDefImpl() 
{
  delete trigSequence;
  delete trigSequence2;
}

ScanDefImpl::ScanDefImpl(const ScanDefImpl &other) :
  trigsPerBurst(other.trigsPerBurst),
  scanPoints(other.scanPoints),
  scanPoints2(other.scanPoints2),
  scanVariable(other.scanVariable),
  scanVariable2(other.scanVariable2),

  trigSequence(new TriggerImpl(*other.trigSequence)),
  trigSequence2(new TriggerImpl(*other.trigSequence2)),

  format(other.format),
  full(other.full), 
  bits32(other.bits32),
  loopCalLine(other.loopCalLine),

  distSlave(other.distSlave),
  debug(other.debug)
{
}

void ScanDefImpl::configure(unsigned short type, float start, float stop, float step)
{
  scanVariable = type;

  int npoints = int(((stop - start) / step) + 1.5);
//   scanPoints.Set(npoints);

  scanPoints.resize(npoints);

  for(int i=0; i<npoints; i++) {
    FLOAT32 point = start + i * step;
    scanPoints[i] = point;
  }
}

void ScanDefImpl::configure2(unsigned short type, float start, float stop, float step)
{
  scanVariable2 = type;

  int npoints = int((stop - start) / step);

  scanPoints2.resize(npoints);

  for(int i=0; i<npoints; i++) {
    FLOAT32 point = start + i * step; 
    scanPoints2[i] = point;
  }
}

const ::SctApi::Trigger *ScanDefImpl::getTrigger1() const
{
  return trigSequence;
}

const ::SctApi::Trigger *ScanDefImpl::getTrigger2() const
{
  return trigSequence2;
}

::SctApi::Trigger *ScanDefImpl::getTrigger1()
{
  return trigSequence;
}

::SctApi::Trigger *ScanDefImpl::getTrigger2()
{
  return trigSequence2;
}

void ScanDefImpl::setTrigger1( ::SctApi::Trigger *trigger)
{
  trigSequence = dynamic_cast< ::SctApi::TriggerImpl *>(trigger);
//   trigSequence.copy(*trigger);
}

void ScanDefImpl::setTrigger2( ::SctApi::Trigger *trigger)
{
  trigSequence2 = dynamic_cast< ::SctApi::TriggerImpl *>(trigger);
//   trigSequence2.copy(*trigger);
}

void ScanDefImpl::setScanVariable1(unsigned short var)
{
  scanVariable = var;
}

void ScanDefImpl::setScanVariable2(unsigned short var)
{
  scanVariable2 = var;
}

unsigned short ScanDefImpl::getScanVariable1() const
{
  return scanVariable;
}

unsigned short ScanDefImpl::getScanVariable2() const
{
  return scanVariable2;
}

void ScanDefImpl::setNTrigs(unsigned long nTrigs) 
{
  trigsPerBurst = nTrigs;
}

unsigned short ScanDefImpl::getNTrigs() const
{
  return trigsPerBurst;
}

const ::SctApi::Scan::ScanPoints ScanDefImpl::getScanPoints1() const
{
  return scanPoints;
}

const ::SctApi::Scan::ScanPoints ScanDefImpl::getScanPoints2() const
{
  return scanPoints2;
}

void ScanDefImpl::setScanPoints1(const ::SctApi::Scan::ScanPoints &scans)
{
  scanPoints = scans;
}

void ScanDefImpl::setScanPoints2(const ::SctApi::Scan::ScanPoints &scans)
{
  scanPoints2 = scans;
}

int ScanDefImpl::getOption(enum ::SctApi::Scan::ScanOptions opt) const
{
  switch(opt) {
  case FORMAT:
    return format;
  case BITS32:
    return bits32;
  case LOOPCALLINE:
    return loopCalLine;
  case DISTSLAVE:
    return distSlave;
  case DEBUG:
    return debug;
  default:
    return 0;
  }
}

void ScanDefImpl::setOption(enum ::SctApi::Scan::ScanOptions opt, int option) 
{
  switch(opt) {
  case FORMAT:
    format = option;
    break;
  case BITS32:
    bits32 = option;
    break;
  case LOOPCALLINE:
    loopCalLine = option;
    break;
  case DISTSLAVE:
    distSlave = option;
    break;
  case DEBUG:
    debug = option;
    break;
  default:
    break;
  }
}

void ScanDefImpl::print() const {
  std::cout << "Scan\n";
  std::cout << "Run: " << getRunNumber() << " Scan: " << getScanNumber() << std::endl;

  std::cout << getNTrigs() << " triggers per scan point\n";

  std::cout << "Histogram format " << (getOption(Scan::FORMAT)?"slice":"block") << std::endl;
  std::cout << "Binning mode " << (getOption(Scan::FULL)?"full":"condensed") << std::endl;
  std::cout << "Bin size " << (getOption(Scan::BITS32)?"32bit":"16bit") << std::endl;
  std::cout << "Cal line loop " << (getOption(Scan::LOOPCALLINE)?"yes":"no") << std::endl;

  std::cout << "Slave DSPs: ";
  switch(getOption(Scan::DISTSLAVE)) {
  case 0: std::cout << "All to Slave 0"; break;
  case 1: std::cout << "According to groups"; break;
  case 2: std::cout << "Router distribution"; break;
  default: std::cout << "Invalid type!"; break;
  }
  std::cout << std::endl;

  if(getOption(Scan::DEBUG)) {
    std::cout << "In debug mode\n";
  }

  if(getScanPoints2().size() > 0) {
    std::cout << "Module set 0\n";
  } else {
    std::cout << "Scan parameters\n";
  }

  std::cout << "Scan over variable number " << getScanVariable1() << std::endl;

  std::cout << getScanPoints1().size() << " scan points:\n";
  int i=0;

  std::vector<FLOAT32> points = getScanPoints1();

  for(std::vector<FLOAT32>::const_iterator iter=points.begin();
      iter != points.end();
      i++, iter++) {
    if(i && (i%20 == 0)) std::cout << std::endl;
    std::cout.width(4);

    std::cout << *iter << " ";
  }
  std::cout << std::endl;

  std::cout << "Trigger:\n";

  getTrigger1()->print();

  if(getScanPoints2().size() > 0) {
    std::cout << "Module set 1\n";

    std::cout << "Scan over variable number " << getScanVariable2() << std::endl;

    std::cout << getScanPoints2().size() << " scan points:\n";
    int i=0;
    for(std::vector<FLOAT32>::const_iterator iter=getScanPoints1().begin();
        iter != getScanPoints1().end();
        i++, iter++) {

      if(i && (i%20 == 0)) std::cout << std::endl;
      std::cout.width(4);

      std::cout << *iter << " ";
    }
    std::cout << std::endl;

    std::cout << "Trigger ";

    Trigger::RODTriggers trigs = getTrigger2()->getRODTriggers();

    for(std::vector<std::pair<UINT16, UINT32> >::const_iterator iter=trigs.begin();
        iter != trigs.end();
        iter++) {
      std::cout.width(4);
      
      std::cout << iter->first << ": " << iter->second << std::endl;
    }

    {
      unsigned short cmd, incr;

      getTrigger2()->getCommIncr(cmd, incr);

      if(incr != 0)
        std::cout << "Increment command " << cmd << " by " << incr << std::endl;
    }
  } else {
    std::cout << "Second trigger not defined\n";
  }

  for(int i=0; i<4; i++) {
    std::cout << "Modules in group " << i << std::endl;

    std::list<std::string> groupList = getModuleList(i);

    for(std::list<std::string>::const_iterator iter = groupList.begin();
        iter != groupList.end();
        iter++) {
      std::cout << " " << *iter << std::endl;
    }
  }
}


void ScanDefImpl::setScanNumber(unsigned int scan) 
{
  scanNumber = scan;
}
 
unsigned int ScanDefImpl::getScanNumber() const 
{
  return scanNumber;
}

void ScanDefImpl::setRunNumber(unsigned int run)
{
  runNumber = run;
}

unsigned int ScanDefImpl::getRunNumber() const 
{
  return runNumber;
}

std::list<std::string> ScanDefImpl::getModuleList(unsigned int group) const 
{
  std::list<std::string> result;

  if(group<groupLists.size()) {
    result = groupLists[group];
  }

  return result;
}
 
void ScanDefImpl::setModuleList(unsigned int group, std::list<std::string> newList)
{
  if(group+1 > groupLists.size()) {
    groupLists.resize(group + 1);
  }
  groupLists[group] = newList;
}

unsigned int ScanDefImpl::getNGroups() const
{
  return m_maxGroup;
}

void ScanDefImpl::setNGroups(unsigned int val)
{
  m_maxGroup = val;
}

void TriggerImpl::print() const 
{
  Trigger::RODTriggers trigs = getRODTriggers();

  std::cout << "Printing trigger: length " << trigs.size() << std::endl;

  if(trigs.size() > 6) {
    std::cout << "Trigger sequence over size\n";
  }

  unsigned short trigData, trigCmd;
  getCommIncr(trigData, trigCmd);

  for(int i = 0; i < trigs.size(); i++) {
    std::cout.width(4);

    std::cout << trigs[i].first << ": " << trigs[i].second << "   ";
    if(trigData != 0 && trigCmd == i) {
      std::cout << "+" << trigData;
    }
    std::cout << std::endl;
  }
}

void ScanDefImpl::copy(const ::SctApi::Scan &scan) 
{
  trigsPerBurst = scan.getNTrigs();

  scanPoints = scan.getScanPoints1();
  scanPoints2 = scan.getScanPoints2();

  scanVariable = scan.getScanVariable1();
  scanVariable2 = scan.getScanVariable2();

  if(!trigSequence) 
    trigSequence = new TriggerImpl;
  trigSequence->copy(*scan.getTrigger1());

  if(!trigSequence2) 
    trigSequence2 = new TriggerImpl;

  trigSequence2->copy(*scan.getTrigger2());

//   setTrigger1(scan.getTrigger1());
//   setTrigger2(scan.getTrigger2());

  setOption(Scan::FORMAT, scan.getOption(Scan::FORMAT));
  setOption(Scan::FULL, scan.getOption(Scan::FULL));
  setOption(Scan::BITS32, scan.getOption(Scan::BITS32));
  setOption(Scan::LOOPCALLINE, scan.getOption(Scan::LOOPCALLINE));
  setOption(Scan::DISTSLAVE, scan.getOption(Scan::DISTSLAVE));
  setOption(Scan::DEBUG, scan.getOption(Scan::DEBUG));
}
