#include "Sct/IS/IOManagerIS.h"
#include "Sct/Exception.h"
#include "Sct/LogicErrors.h"
#include "Sct/SctNames.h"
#include "Sct/SctParameters.h"
#include "SctData/NoiseOccupancyTestResult.h"
#include <is/isinfo.h>
#include <boost/shared_ptr.hpp>
#include <iostream>

using namespace Sct;
using namespace Sct::IS;
using namespace boost;
using namespace SctData;
using namespace std;

string testName = "TestData.SctData::NoiseOccupancyTestResult.551.54.SCTTestAPI_PseudoModule";
double compData[] = {1469, 1440, 1482, 1453, 1426, 1453, 1459, 1462, 1451, 1441, 1412, 1461};
double tolerance = 2;
bool debug = false;

void waitFor(string name, int nsecs) {
    int tries = 0;
    while (!SctNames::getISDictionary().contains(name.c_str()) && tries < nsecs) {
	if (!sleep(1))
	    ++tries;
    }
    if (tries == nsecs) {
	throw IllegalStateError("Didn't wait long enough", __FILE__, __LINE__);
    }
    if (debug) {
	cout << "Waited for " << tries << " secs" << endl;
    }
}

bool compare(shared_ptr<NoiseOccupancyTestResult> result) {    
    bool retVal = true;
    for (unsigned int i=0; i<nChipModule; ++i) {
	if (fabs(result->getChipResult(i).getNoise() - compData[i]) > tolerance) {
	    cout << "Comparison failed at chip: " << i << " Noise: " << result->getChipResult(i).getNoise() << " wanted: " << compData[i] << " to: " << tolerance << endl;
	    retVal = false;
	} else if (debug) {
	    cout << "Comparison successful for chip: " << i << " Noise: " << result->getChipResult(i).getNoise() << " SCTDAQ noise: " << compData[i] << " Tolerance: " << tolerance << endl;
	}
    }
    return retVal;
}

int main(int argc, char** argv) {
    Sct::setExceptionHandlers(argv[0]);
    try {
	waitFor(testName, 20);
	shared_ptr<Serializable> ob = IOManagerIS::instance().read(testName);
	shared_ptr<NoiseOccupancyTestResult> result = dynamic_pointer_cast<NoiseOccupancyTestResult>(ob);
	if (!result) 
	    throw IllegalStateError("Failed to read test result: " + testName, __FILE__, __LINE__);
	if (compare(result)) {
	    if (debug) cout << "Success!" << endl;
	    return 0;
	}
	else {
	    cout << "Failed" << endl;
	    return 1;
	}
	
    } catch (Throwable& e) {
	e.sendToMrs(MRS_FATAL);
	return 1;
    }
}
