#include "Sct/IS/IOManagerIS.h"
#include "Sct/Exception.h"
#include "Sct/LogicErrors.h"
#include "Sct/SctNames.h"
#include "Sct/SctParameters.h"
#include "SctData/TrimRangeTestResult.h"
#include <is/isinfo.h>
#include <boost/shared_ptr.hpp>
#include <iostream>
#include <fstream>

#include "TGraph.h"

using namespace Sct;
using namespace Sct::IS;
using namespace boost;
using namespace SctData;
using namespace std;

string filename = "20220330200020_tr-1_20030708.trim";
string testName = "TestData.SctData::TrimRangeTestResult.551.16.SCTTestAPI_PseudoModule";
double compData[] = {};
bool debug = false;

int range[12];
double target[12];
int trim[12][128];

void waitFor(string name, int nsecs) {
    ISInfoDictionary d(SctNames::getPartition());
    int tries = 0;
    while (!d.contains(name.c_str()) && tries < nsecs) {
	if (!sleep(1))
	    ++tries;
    }
    if (tries == nsecs) {
	throw IllegalStateError("Didn't wait long enough", __FILE__, __LINE__);
    }
    if (debug) {
	cout << "Waited for " << tries << " secs" << endl;
    }
}

void loadData(){
    cout << "Loading trim data from file " << filename << endl;
    ifstream f(filename.c_str());
    if (f.fail()) throw FileException(filename, "Error in opening Trimdata file", __FILE__, __LINE__);
    string dummy_string;
    for (unsigned i=0; i<12; ++i){
	f >> dummy_string >> dummy_string;
	f >> range[i] >> target[i];
    }

    float temp_float;
    for (unsigned ichip=0; ichip<12; ichip++){
	for (unsigned ichan=0; ichan<128; ichan++){
	    f >> temp_float;
	    temp_float *= 15;
	    trim[ichip][ichan]=(int)(temp_float+0.01);
	}
    }
}

bool compare(shared_ptr<TrimRangeTestResult> result) {    
    bool retVal = true;
    
    for (unsigned ichip=0; ichip<12; ++ichip){
	const TrimRangeTestResult::ChipTrim& t = *result->chipTrim[ichip];
	if (t.range != range[ichip]){
	    cout << "CHIP " << ichip << " range is " 
		 << t.range
		 << " SCTDAQ: " << range[ichip] << endl;
	    retVal=false;
	}
	if (fabs(t.target-target[ichip])>4.0){
	    cout << "CHIP " << ichip 
		 << " target is " << t.target
		 << " SCTDAQ: " << target[ichip] << endl;
	    retVal=false;
	}
    }
    
    
    unsigned trimdiff[3]={0,0,0};
    for (unsigned ichip=0; ichip<12; ++ichip){
	const TrimRangeTestResult::ChipTrim& t = *result->chipTrim[ichip];
	for (unsigned ichan=0; ichan<128; ++ichan){
	    short int our_trim=t.channelTrim.getAt(ichan).value.trim;
	    
//	    t.target+
	    
	    int diff = abs(our_trim - trim[ichip][ichan]);
	    
	    if (diff!=0){
		cout << "CHIP " << ichip 
		     << " CHAN " << ichan 
		     << " trim is " << our_trim
		     << " SCTDAQ: " << trim[ichip][ichan] << endl;
		for (int idiff=0; idiff<3; ++idiff){
		    if (diff>idiff) trimdiff[idiff]++;
		}
	    }
	}
    }
    for (int idiff=0; idiff<3; ++idiff){
	cout << "#channels with trim different by more than " 
	     << idiff << " = " << trimdiff[idiff] << endl;
	
    }

    if (trimdiff[1]>3 || trimdiff[2]>1) retVal=false;
    return retVal;
}

int main(int argc, char** argv) {
    Sct::setExceptionHandlers(argv[0]);
    try {
	loadData();
	waitFor(testName, 60);
	shared_ptr<Serializable> ob = IOManagerIS::instance().read(testName);
	shared_ptr<TrimRangeTestResult> result = dynamic_pointer_cast<TrimRangeTestResult>(ob);
	if (!result) 
	    throw IllegalStateError("Failed to read test result: " + testName, __FILE__, __LINE__);
	if (compare(result)) {
	    if (debug) cout << "Success!" << endl;
	    return 0;
	}
	else {
	    cout << "Failed" << endl;
	    return 1;
	}
	
    } catch (Throwable& e) {
	e.sendToMrs(MRS_FATAL);
	return 1;
    }
}
