package SctApi.conf.gui;

import sctConf.*;
import ipc.*;

import java.util.Enumeration;
import java.util.Vector;
import java.awt.Dimension;
import java.awt.BorderLayout;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.TreeModelListener;
import javax.swing.event.TreeModelEvent;

import org.omg.CORBA.IntHolder;

public class ConfigGui extends JFrame {
    private sctConf.Configuration conf;

    final private ConfigGui finalGui;

    public static void main(String args[]) {
        System.out.println("Hello world");

        JFrame frame = new ConfigGui();

        frame.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });  

        frame.pack();
        frame.setVisible(true);
    }

    public ConfigGui() {
        super("Configuration GUI");

        finalGui = this;

        Partition p= new Partition("SCT");
        try {
            org.omg.CORBA.Object obj = p.lookup("SCTConfigurationServer", "Test_Server"); 
            if (obj == null) System.out.println("Object SCTConfigurationServer not found in server Test_Server");
            conf = sctConf.ConfigurationHelper.narrow(obj);
        } catch (org.omg.CORBA.BAD_PARAM bp) {
            System.err.println("Couldn't get api object: " + bp);
        } catch (org.omg.CORBA.COMM_FAILURE cf) {
            System.err.println("CORBA communications error: " + cf);
        } catch (java.lang.Exception e) {
            System.err.println("Other error" + e);
        }

	if (conf==null) {
	    System.err.println("Error (check configuration running)");
	    System.exit(0);
	}
        System.out.println("Found configuration");

        JPanel buttons = new JPanel();

        JButton close = new JButton("Close");
        close.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    dispose();
                }
            });
        buttons.add(close);

        JButton physic = new JButton("View Physical");
        physic.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    JFrame frame = new PhysicalView(conf);

                    frame.addWindowListener(new WindowAdapter() {
                            public void windowClosing(WindowEvent e) {
                                e.getWindow().dispose();
                            }
                        });
                    frame.pack();
                    frame.setVisible(true);
                }
            });
        buttons.add(physic);

        JButton myTreeView = new JButton("Tree View");
        myTreeView.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    JFrame frame = new TreeView(finalGui, conf);

                    frame.addWindowListener(new WindowAdapter() {
                            public void windowClosing(WindowEvent e) {
                                e.getWindow().dispose();
                            }
                        });
                    frame.pack();
                    frame.setVisible(true);
                }
            });
        buttons.add(myTreeView);

        JButton moduleListView = new JButton("Module List");
        moduleListView.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    JFrame frame = new ModuleListView(conf);

                    frame.addWindowListener(new WindowAdapter() {
                            public void windowClosing(WindowEvent e) {
                                e.getWindow().dispose();
                            }
                        });
                    frame.pack();
                    frame.setVisible(true);
                }
            });
        buttons.add(moduleListView);

        JButton shutdown = new JButton("Shutdown server");
        shutdown.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    try {
                        System.out.println("Dump configuration");
                        conf.saveConfiguration("Shutdown_Dump_From_ConfigGUI.xml");
                        System.out.println("Done");
                    } catch(ConfigurationException c) {
                        System.out.println("Shutdown xml dump failed");
                    }
                    conf.destroy();
                    dispose();
                    System.exit(0);
                }
            });
        buttons.add(shutdown);

        JButton saveConf = new JButton("Save configuration");
        saveConf.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    try {
                        conf.saveConfiguration("Dump_From_ConfigGUI.xml");
                    } catch(ConfigurationException c) {
                        System.out.println("Save configuration failed!");
                    }
                }
            });
        buttons.add(saveConf);

        JButton loadConf = new JButton("Reload configuration");
        loadConf.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    try {
                        conf.loadConfiguration("");
                    } catch(ConfigurationException c) {
                        System.out.println("Load configuration failed!");
                    }
                }
            });
        buttons.add(loadConf);

        getContentPane().add(buttons, BorderLayout.SOUTH);

        System.out.println("Set up GUI, wait for it to run");
    }

    public MyPartition getPartition(int i) {
        return new MyPartition(i);
    }

    public class MyPartition { // implements ActionListener, TreeNode {
        private int m_i;

        MyPartition(int i) { m_i = i; }

        public int index() { return m_i; }
        public String toString() {
            return "Partition " + m_i;
        }
    }

    public Crate getCrate(int i, int part) {
        return new Crate(i, part);
    }

    public class Crate {
        int m_i;
        int p;
        Crate(int i, int part) { m_i = i; p = part; }
        public int index() { return m_i; }
        public int partition() { return p; }
        public String toString() {
            return "Crate " + p + " " + m_i;
        }
    }

    public Rod getRod(int i, int part, int crate) {
        return new Rod(i, part, crate);
    }

    public class Rod {
        int m_i;
        int p, c;
        Rod(int i, int crate, int part) { m_i = i; c = crate; p = part; }
        public int partition() { return p; }
        public int crate() { return c; }
        public int index() { return m_i; }
        public String toString() {
            return "Rod " + p + " " + c + " " + m_i;
        }
    }

    public MUR getMUR(int i, int mur, int part, int crate, int rod) {
        return new MUR(i, mur, part, crate, rod);
    }

    public class MUR {
        int m_MUR, m_i;
        int m_p, m_c, m_r;
        MUR(int i, int mur, int partition, int crate, int rod) { m_MUR = mur; m_i = i; m_c = crate; m_p = partition; m_r = rod; }
        public int partition() { return m_p; }
        public int crate() { return m_c; }
        public int rod() { return m_r; }
        /// Index within ROD
        public int index() { return m_i; }
        /// Global ID
        public int id() { return m_MUR; }
        public String toString() {
            return "MUR " + m_MUR + " at " + m_i + " in " + m_p + " " + m_c + " " + m_r;
        }
    }

    public Module getModule(String sn) {
        return new Module(sn);
    }

    public class Module {
        String sn;
        Module(String sNum) { sn = sNum; }
        public String toString() {
            return sn;
        }
        public String getSN() {
            return sn;
        }
        public String getMUR() {
            try {
                IntHolder MUR = new IntHolder(), num = new IntHolder();
                conf.translateFromSN(sn, MUR, num);
                return "MUR: " + MUR.value + " index: " + num.value;
            } catch(ConfigurationException e) {
                return "No mapping to MUR";
            }
        }

        public String getPower() {
            try {
                IntHolder MUR = new IntHolder(), num = new IntHolder();
                conf.translateFromSN(sn, MUR, num);

                IntHolder partition = new IntHolder(), crate = new IntHolder(), 
                    channel = new IntHolder();
                conf.translateToPowerSupply(MUR.value, num.value, partition, crate, channel);
                return "Power: " + partition.value + " " + crate.value + " " + channel.value;
            } catch(ConfigurationException e) {
                return "Power mapping undefined";
            }
        }

        public String getROD() {
            try {
                IntHolder MUR = new IntHolder(), num = new IntHolder();
                conf.translateFromSN(sn, MUR, num);

                IntHolder partition = new IntHolder(), crate = new IntHolder(), 
                    rod = new IntHolder(), channel = new IntHolder();
                conf.translateToROD(MUR.value, num.value, 
                                    partition, rod, crate, channel);
                return "Partition: " + partition.value + " Crate: " + crate.value 
                    + " ROD: " + rod.value + " Channel: " + channel.value;
            } catch(ConfigurationException e) {
                return "No mapping for ROD";
            }
        }
        public String getBarrel() {
            try {
                IntHolder MUR = new IntHolder(), num = new IntHolder();
                conf.translateFromSN(sn, MUR, num);

                IntHolder barrel = new IntHolder(), row = new IntHolder(), number = new IntHolder();
                conf.translateToBarrel(MUR.value, num.value, 
                                       barrel, row, number);
                return "Barrel: " + barrel.value + " Row: " + row.value + " Number: " + number.value;
            } catch(ConfigurationException e) {
                return "No mapping for Barrel";
            }
        }
    }
}
