package SctApi.conf.gui;

import java.util.Enumeration;
import java.util.Vector;
import java.awt.Dimension;
import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import javax.swing.Box;
import javax.swing.JDialog;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.tree.*;
import javax.swing.event.TreeModelListener;
import javax.swing.event.TreeModelEvent;

import org.omg.CORBA.IntHolder;

import sctConf.*;
import ipc.*;

public class TreeView extends JFrame {
    private ConfigGui configGui;
    private sctConf.Configuration conf;

    private JTree tree;

    private final JFrame treeFrame;

    private JPopupMenu popup;

    private MyModel treeModel;

    private MURSelectDialog dialog;

    public TreeView(ConfigGui cGui, Configuration config) {
        super("Tree view");

        configGui = cGui;
        conf = config;

        treeModel = new MyModel(conf);

        tree = new JTree(treeModel);

        treeFrame = this;
        JScrollPane treeView = new JScrollPane(tree);

        enableEvents(java.awt.AWTEvent.WINDOW_EVENT_MASK);
        tree.addMouseListener(new MouseClickHandler());

        dialog = new MURSelectDialog(this); 

        popup = new JPopupMenu();
        popup.add(new JMenuItem("Do nothing"));

        treeView.setPreferredSize(new Dimension(500, 300));
        getContentPane().add(treeView, BorderLayout.CENTER);

        System.out.println("Set up GUI, wait for it to run");
    }

    private class MouseClickHandler extends MouseAdapter {
        private JMenuItem menuWithAction(String name, Object thing) {
            JMenuItem result = new JMenuItem(name);
            result.addActionListener(new ActionHandler(name, thing));

            return result;
        }

        private void checkPopup(MouseEvent event) {
            TreePath path = tree.getPathForLocation(event.getX(), event.getY());
            if(path != null){
                tree.setSelectionPath(path);
                if(event.isPopupTrigger()){
                    Object thing = path.getLastPathComponent();

                    System.out.println("Last in path is: " + thing);

                    try {
                        while(true) {
                            popup.remove(0);
                        }
                    } catch(IllegalArgumentException e) {
                        // There must be a better way of doing this!
                    }

                    if(thing == null) {
                    } else if(thing == "Partitions") {
                        popup.add(menuWithAction("Clear everything", thing));
                    } else if(thing instanceof ConfigGui.MyPartition) { 
                        popup.add(menuWithAction("Clear crates", thing));
                        popup.add(menuWithAction("Add crate to Partition", thing));
                    } else if(thing instanceof ConfigGui.Crate) { 
                        popup.add(menuWithAction("Clear RODs in crate", thing));
                        popup.add(menuWithAction("Add ROD to crate", thing));
                    } else if(thing instanceof ConfigGui.Rod) { 
                        popup.add(menuWithAction("View ROD info", thing));
//                         popup.add(menuWithAction("Add Module to ROD", thing));
                    } else if(thing instanceof ConfigGui.MUR) { 
                        popup.add(menuWithAction("Add Module to MUR", thing));
                    } else if(thing instanceof ConfigGui.Module) { 
                        popup.add(menuWithAction("View module info", thing));
                        popup.add(menuWithAction("Edit module mappings", thing));
                    } else if(thing instanceof String) { 
                        String name = (String)thing;
                    }
                    if(popup.getComponents().length > 0) {
                        popup.show(tree, event.getX(), event.getY());
                    }
                }
            }
        }
        public void mousePressed(MouseEvent event){
            checkPopup(event);
        }
        public void mouseReleased(MouseEvent event){
            checkPopup(event);
        }
    }   

    private class ActionHandler implements ActionListener {
        Object target;
        public ActionHandler(String name, Object thing) {
            target = thing;
        }

        public void actionPerformed(ActionEvent e) {
            String action = e.getActionCommand();

            System.out.println("Do " + action + " on " + target);

            if(target instanceof ConfigGui.Crate) {
                ConfigGui.Crate crate = (ConfigGui.Crate)target;
                if(action == "Clear RODs in crate") {
                    System.out.println("Don't know that one yet");
                } else if(action == "Add ROD to crate") {
                    try {
                        int [] RODs = conf.listRodsInCrate(crate.partition(), crate.index());
                        RodConfig rConf = new RodConfig();

                        rConf.slaves = new sctConf.SlaveConfig[4];
                        for(int i=0; i<4; i++) {
                            rConf.slaves[i] = new SlaveConfig();
                            rConf.slaves[i].emifFile = "";
                            rConf.slaves[i].ipramFile = "";
                            rConf.slaves[i].idramFile = "";
                            rConf.slaves[i].extFile = "";
                        }

                        conf.configureROD(crate.partition(), crate.index(), RODs.length, rConf);

                        Object [] path = new Object[3];

                        path[0] = "Partitions";
                        path[1] = configGui.getPartition(crate.partition());
                        path[2] = target;

                        treeModel.modified(path);
                    } catch(ConfigurationException c) {
                        System.out.println("ROD creation failed");
                    }
                } else {
                    System.out.println("Unknown action " + action + " on Crate " + crate);
                }
            } else if(target instanceof ConfigGui.Rod) {
                ConfigGui.Rod r = (ConfigGui.Rod)target;
                if(action == "View ROD info") {
                    JFrame frame = new RODView(conf, r);

                    frame.addWindowListener(new WindowAdapter() {
                            public void windowClosing(WindowEvent e) {
                                e.getWindow().dispose();
                            }
                        });
                    frame.pack();
                    frame.setVisible(true);
                } else if(action == "Add Module to ROD") {
                    // Need an MUR a position and a serial number 
                    // Would also like RMUR !
                    int [] myConf;
                    myConf = new int[100];

//                     MURSelectDialog dia = new MURSelectDialog(this);

                    System.out.println("Adding ROD " + r);
                    dialog.setROD(r); // (ConfigGui.Rod)target);
                    dialog.show(); 

                    System.out.println("Got back some selections " + dialog.getMyMUR() + " " + dialog.getMyPos() + " " + dialog.getMySN());

                    try {
                        //conf.createModule(dialog.getMyMUR(), dialog.getMyPos(), dialog.getMyMUR(), dialog.getMyPos(), dialog.getMySN(), myConf); 
			throw new ConfigurationException("Can't create Module - feature removed!");
		    } catch(ConfigurationException ex) {
                        System.out.println("CreateModule failed with " + ex);
                    }
                } else {
                    System.out.println("Unknown action " + action + " on ROD ");
                }
            } else if(target instanceof ConfigGui.MUR) {
                ConfigGui.MUR mur = (ConfigGui.MUR)target;
                if(action == "Add Module to MUR") {
                    String moduleName = null;
                    int position = -1;
                    {
                        ModuleListView moduleList = new ModuleListView(conf);

                        Vector list = moduleList.getList();

                        if(list.size() == 0) {
                            System.out.println("No modules to select");
                        } else {
                            Object [] objList = new Object[list.size()];

                            list.copyInto(objList);

                            String item = (String)JOptionPane.showInputDialog((JComponent)(e.getSource()),
                                                                              "Select an unused module",
                                                                              "Module selector",
                                                                              JOptionPane.OK_CANCEL_OPTION,
                                                                              null, // JOptionPane.PLAIN_MESSAGE,
                                                                              objList,
                                                                              objList[0]);

                            System.out.println("Found a module! " + item); //  + ": " + list.get(item));

                            moduleName = item;
                        }
                    }

                    try {
                        String [] modules = conf.listModulesInMUR(mur.partition(), mur.id());

                        int [] check = new int[6];
                        for(int i=0; i<modules.length; i++) {
                            IntHolder MUR = new IntHolder(), mod = new IntHolder();
                            conf.translateFromSN(modules[i].toString(), MUR, mod);
                            check[mod.value-1] = 1;
                        }

                        Integer [] numList = new Integer[6 - modules.length];
                        int index = 0;
                        for(int i = 1; i<=6; i++) {
                            if(check[i-1] == 0) {
                                numList[index++] = new Integer(i);
                            }
                        }

                        if(modules.length < 6) {
                            Integer item = (Integer)JOptionPane.showInputDialog((JComponent)(e.getSource()),
                                                                                "Select an unused position",
                                                                                "Position selector",
                                                                                JOptionPane.OK_CANCEL_OPTION,
                                                                                null, 
                                                                                numList,
                                                                                numList[0]);

                            position = item.intValue();
                        } else {
                            System.out.println("No positions free");
                        }

                        System.out.println("Got position " + position);
                    } catch(ConfigurationException c) {
                        System.out.println("Bad configuration for getting MUR list");
                    }

                    if(moduleName != null && position != -1) {
                        System.out.println("Mapping module " + moduleName + " in MUR " + mur.id() + " to number " + position);
                        try {
                            conf.mapModuleMUR(mur.id(), position, mur.id(), (position%6)+1, moduleName);

                            Object [] path = new Object[5];

                            path[0] = "Partitions";
                            path[1] = configGui.getPartition(mur.partition());
                            path[2] = configGui.getCrate(mur.crate(), mur.partition());
                            path[3] = configGui.getRod(mur.rod(), mur.partition(), mur.crate());
                            path[4] = target;

                            for(int i=0; i<5; i++) {
                                System.out.println(i + ": " + path[i]);
                            }

                            treeModel.modified(path);
                        } catch(ConfigurationException c) {
                            System.out.println("Failed to map module");
                        }
                    }
                } else {
                    System.out.println("Unknown action " + action + " on MUR " + mur);
                }
            } else if(target instanceof ConfigGui.Module) {
                ConfigGui.Module module = (ConfigGui.Module)target;
                if(action == "View module info") {
                    JFrame frame = new ModuleView(conf, module);

                    frame.addWindowListener(new WindowAdapter() {
                            public void windowClosing(WindowEvent e) {
                                e.getWindow().dispose();
                            }
                        });

                    frame.pack();

                    frame.setVisible(true);
                } else {
                    System.out.println("Unknown action " + action + " on module " + module);
                }
            } else if(target == "Partitions") { 
                System.out.println("Permission denied!");
            } else {
                System.out.println("Can't do that yet!");
            }
        }
    }

    public class MyModel implements TreeModel { // extends DefaultTreeModel {
        Configuration conf;

        java.util.Vector listeners;

        public MyModel(Configuration c) {
            //            super(new DefaultMutableTreeNode("Partitions"));
            conf = c;
            listeners = new java.util.Vector();
        }

        public Object getRoot() {
            return "Partitions";
        }

        public Object getChild(Object parent, int index) {
            System.out.print("getChild");

            Object ret = null;
            try {
                if(parent == "Partitions") {
                    ret = configGui.getPartition(index);
                } else if(parent instanceof ConfigGui.MyPartition) {
                    ConfigGui.MyPartition p = (ConfigGui.MyPartition)parent;
                    ret = configGui.getCrate(index, p.index());
                } else if(parent instanceof ConfigGui.Crate) {
                    ConfigGui.Crate c = (ConfigGui.Crate)parent;
                    int[] RODs = conf.listRodsInCrate(c.partition(), c.index());
                    ret = configGui.getRod(RODs[index], c.index(), c.partition());
                } else if(parent instanceof ConfigGui.Rod) {
                    ConfigGui.Rod r = (ConfigGui.Rod)parent;
                    int[] MURs = conf.listMURSInRod(r.partition(), r.crate(), r.index());
                    ret = configGui.getMUR(index, MURs[index], r.partition(), r.crate(), r.index());
                } else if(parent instanceof ConfigGui.MUR) {
                    ConfigGui.MUR mur = (ConfigGui.MUR)parent;

                    System.out.print(" Asked for child " + index + " of MUR " + mur);

                    int moduleCount = 0;
                    String[] modules = conf.listModulesInMUR(mur.partition(), mur.id());

                    ret = configGui.getModule(modules[index]);
                } else if(parent instanceof ConfigGui.Module) {
                    ConfigGui.Module m = (ConfigGui.Module)parent;
                    IntHolder MUR = new IntHolder(), num = new IntHolder();
                    conf.translateFromSN(m.toString(), MUR, num);
                    switch(index) {
                    case 0:
                        {
                            try {
                                IntHolder barrel = new IntHolder(), row = new IntHolder(), number = new IntHolder();
                                conf.translateToBarrel(MUR.value, num.value, 
                                                       barrel, row, number);
                                ret = "Barrel " + barrel.value + " Row " + row.value + " Number " + number.value;
                            } catch(ConfigurationException e) {
                                ret = "No barrel mapping defined";
                            }
                            break;
                        }
                    case 1:
                        {
                            try {
                                IntHolder partition = new IntHolder(), crate = new IntHolder(), 
                                    rod = new IntHolder(), channel = new IntHolder();
                                conf.translateToROD(MUR.value, num.value, 
                                                    partition, rod, crate, channel);
                                ret = partition.value + " " + rod.value + " " + crate.value + " " + channel.value;
                            } catch(ConfigurationException e) {
                                ret = "No ROD mapping defined";
                            }
                            break;
                        }
                    case 2:
                        {
                            try {
                                IntHolder partition = new IntHolder(), crate = new IntHolder(), 
                                    channel = new IntHolder();
                                conf.translateToPowerSupply(MUR.value, num.value, 
                                                            partition, crate, channel);
                                ret = "Power: " + partition.value + " " + crate.value + " " + channel.value;
                            } catch(ConfigurationException e) {
                                ret = "No power mapping defined";
                            }
                            break;
                        }
                    case 3:
                        {
                            ret = "MUR " + MUR.value + ": " + num.value;
                            break;
                        }
                    case 4:
                        {
                            try {
                                conf.getModuleConfig(m.getSN());
                                ret = "Module Configuration defined";
                            } catch(ConfigurationException e) {
                                ret = "No module configuration defined!";
                            }
                            break;
                        }
                    default:
                    }
                } else if(parent instanceof String) {
                    // Default?
                }
            } catch(ConfigurationException e) {
                ret = "Exception getting result";
            }

            System.out.println(".");
            return ret;
        }

        public int getChildCount(Object parent) {
            System.out.print("getChildCount");

            int ret = 0;
            try {
                if(parent == null) {
                    ret = 0;
                } else if(parent == "Partitions") {
                    int[] parts = conf.listPartitions();
                    ret = parts.length;
                } else if(parent instanceof ConfigGui.MyPartition) { 
                    ConfigGui.MyPartition p = (ConfigGui.MyPartition)parent;
                    int[] crates = conf.listCratesInPartition(p.index());
                    ret = crates.length;
                } else if(parent instanceof ConfigGui.Crate) { 
                    ConfigGui.Crate c = (ConfigGui.Crate)parent;
                    int[] rods = conf.listRodsInCrate(c.partition(), c.index());
                    ret = rods.length;
                } else if(parent instanceof ConfigGui.Rod) {
                    ConfigGui.Rod r = (ConfigGui.Rod)parent;
                    int[] MURs = conf.listMURSInRod(r.partition(), r.crate(), r.index());
                    ret = MURs.length;
                } else if(parent instanceof ConfigGui.MUR) {
                    ConfigGui.MUR mur = (ConfigGui.MUR)parent;

                    ret = conf.listModulesInMUR(mur.partition(), mur.id()).length;
                } else if(parent instanceof ConfigGui.Module) {
                    ret = 5;   // Number of mappings listed under a module
                } else if(parent instanceof String) {
                    ret = 0;
                } 
            } catch(ConfigurationException e) {
                ret = 0;
            }

            System.out.println(" " + ret + ".");
            return ret;
        }

        public int getIndexOfChild(Object parent,
                                   Object child) {
            System.out.print("getIndexOfChild");
            int ret = -1;

            try {
                if(parent == null) {
                    ret = -1;
                } else if(parent == "Partitions") {
                    int[] parts = conf.listPartitions();
                    int partition = ((ConfigGui.MyPartition)child).index();
                    for(int i=0; i<parts.length; i++) {
                        if(parts[i] == partition) 
                            ret = i;
                    }
                } else if(parent instanceof ConfigGui.MyPartition) { 
                    ConfigGui.Crate c = (ConfigGui.Crate)child;
                    ConfigGui.MyPartition p = (ConfigGui.MyPartition)parent;
                    int[] crates = conf.listCratesInPartition(p.index());

                    int crate = c.index();
                    for(int i=0; i<crates.length; i++) {
                        if(crates[i] == crate) 
                            ret = i;
                    }
                } else if(parent instanceof ConfigGui.Crate) { 
                    ConfigGui.Rod r = (ConfigGui.Rod)child;
                    ConfigGui.Crate c = (ConfigGui.Crate)parent;
                    int[] rods = conf.listRodsInCrate(c.partition(), c.index());

                    int rod = r.index();
                    for(int i=0; i<rods.length; i++) {
                        if(rods[i] == rod) 
                            ret = i;
                    }
                } else if(parent instanceof ConfigGui.Rod) { 
                    ConfigGui.MUR mur = (ConfigGui.MUR)child;
                    ConfigGui.Rod r = (ConfigGui.Rod)parent;
                    int[] murs = conf.listMURSInRod(r.partition(), r.crate(), r.index());

                    int murId = mur.id();
                    for(int i=0; i<murs.length; i++) {
                        if(murs[i] == murId) 
                            ret = i;
                    }
                } else if(parent instanceof ConfigGui.MUR) { 
                    ConfigGui.Module module = (ConfigGui.Module)child;
                    ConfigGui.MUR mur = (ConfigGui.MUR)parent;
                    String[] modules = conf.listModulesInMUR(mur.partition(), mur.index());

                    String sn = module.getSN();
                    for(int i=0; i<modules.length; i++) {
                        if(modules[i] == sn) 
                            ret = i;
                    }

                    IntHolder MUR = new IntHolder(), mod = new IntHolder();
                    conf.translateFromSN(module.toString(), MUR, mod);

                    System.out.println("translateFromSN returned MUR " + MUR.value + " Stored is " + mur.id());

//                     ret = mod.value;
                } else if(parent instanceof ConfigGui.Module) { 
                    ret = 0; 

                    System.out.println("Count strings...");

//                     ConfigGui.Module module = (ConfigGui.Module)parent;
//                     IntHolder MUR = new IntHolder(), mod = new IntHolder(), 
//                         part = new IntHolder(), cr = new IntHolder(), rod = new IntHolder(), ch = new IntHolder();
//                     conf.translateFromSN(module.toString(), MUR, mod);
//                     conf.translateToROD(MUR.value, mod.value, part, cr, rod, ch);
                
//                     int[] MURs = conf.listMURSInRod(part.value, cr.value, rod.value);
                
//                     int moduleCount = 0;
//                     outer: 
//                     for(int i=0; i<MURs.length; i++) {
//                         String [] modules = conf.listModulesInMUR(part.value, MURs[i]);
//                         for(int m=0; m<modules.length; m++) {
//                             if(modules[m] == module.toString()) {
//                                 ret = moduleCount + m;
//                                 break outer;
//                             }
//                         }
//                     }
                } else if(parent instanceof String) { 
                    //                String name = (String)parent;

                    ret = -1;
                } 
            } catch(ConfigurationException e) {
                ret = 0;
            }
            System.out.println(" " + ret + ".");
            return ret;
        }

        public boolean isLeaf(Object node) {
            if(node instanceof String) {
                if(node != "Partitions") {
                    return true;
                }
            }
            return false;
        }

        public void valueForPathChanged(TreePath path,
                                        Object newValue) {
        }

        public void addTreeModelListener(TreeModelListener l) {
            System.out.println("Add listener " + l);
            listeners.add(l);
        }

        public void removeTreeModelListener(TreeModelListener l) {
            System.out.println("Remove listener " + l);
            listeners.remove(l);
        }

        public void modified(Object o) {
            for(Enumeration e = listeners.elements(); e.hasMoreElements(); ) {
                TreeModelListener t = (TreeModelListener)e.nextElement();

                Object [] path = new Object[1];

                path[0] = "Partitions";

                TreeModelEvent ev = new TreeModelEvent(this, path);

                t.treeStructureChanged(ev);
            }
        }

        public void modified(Object [] path) {
            for(Enumeration e = listeners.elements(); e.hasMoreElements(); ) {
                TreeModelListener t = (TreeModelListener)e.nextElement();

                TreeModelEvent ev = new TreeModelEvent(this, path);

                t.treeStructureChanged(ev);
            }
        }
    }

    public class MURSelectDialog extends JDialog {
        ConfigGui.Rod myRod;

        MURSelectDialog(java.awt.Frame owner) {
            super(owner, "Select MUR and position", true);

            Box box = Box.createVerticalBox();

            box.add(new JLabel("" + myRod));

            getContentPane().add(box);
        }

        void setROD(ConfigGui.Rod newRod) {
            myRod = newRod;
        }

        int getMyPos() {
            return 5;
        }

        String getMySN() {
            return "2022018000000";
        }

        int getMyMUR() {
//             ConfigGui.Rod r = (ConfigGui.Rod)parent;
            try {
                int[] MURs = conf.listMURSInRod(myRod.partition(), myRod.crate(), myRod.index());
                return MURs[0];
            } catch(ConfigurationException e) {
                System.out.println("No MURs in ROD? " + e);
            }

            return 0;
        }
    }
}
