#include <stdlib.h>
#include <ipc/object.h>
#include <ipc/server.h>

#include "sctConf.hh"
#include "configserver.h"
#include "Sct/ILUMarshalling.h"
#include "marshalling.h"

using namespace Sct;
using namespace SctConfiguration;

typedef sctConf_T_ilu__prefix_idlExceptionType_ConfigurationException ConfException;

static void myRaise(sctConfStatus *_status, ConfigurationException &c);

using namespace std;

ConfigServer::ConfigServer(char* oid, IPCServer* server) : IPCObject(oid, server) 
{
    ///This fails if sctStructure has been updated without updating the definition of ABCDModule in configuration.idl
    assert(sizeof(sctConf_T_ABCDModule) == sizeof(ABCDModule));
}

ConfigServer::~ConfigServer() 
{
}

sctConf_T_PartitionList ConfigServer::listPartitions (sctConfStatus *_status) 
{
  sctConf_T_PartitionList result;

  result = _sctConf_T_PartitionList_sequence::Create(4, NULL);
  result->Clear(0);

  list<unsigned int> parts = config.listPartitions();

  for(list<unsigned int> :: const_iterator p = parts.begin();
      p!=parts.end();
      p++) {
    result->Append(*p);
  }

  return result;
}

sctConf_T_CrateList ConfigServer::listCratesInPartition (sctConfStatus *_status, ilu_Cardinal partition) 
{
  sctConf_T_CrateList result;

  result = _sctConf_T_CrateList_sequence::Create(4, NULL);
  result->Clear(0);

  list<unsigned int> crates = config.listCratesInPartition(partition);

  for(list<unsigned int> :: const_iterator c = crates.begin();
      c!=crates.end();
      c++) {
    result->Append(*c);
  }

  return result;
}

sctConf_T_RodList ConfigServer::listRodsInCrate (sctConfStatus *_status, ilu_Cardinal partition, ilu_Cardinal crate) 
{
  sctConf_T_RodList result;

  result = _sctConf_T_RodList_sequence::Create(4, NULL);
  result->Clear(0);

  list<unsigned int> rods = config.listRodsInCrate(partition, crate);

  for(list<unsigned int> :: const_iterator r = rods.begin();
      r!=rods.end();
      r++) {
    result->Append(*r);
  }

  return result;
}

sctConf_T_MURList ConfigServer::listMURSInRod (sctConfStatus *_status, ilu_Cardinal partition, ilu_Cardinal crate, ilu_Cardinal rod) 
{
  sctConf_T_MURList result;

  result = _sctConf_T_MURList_sequence::Create(4, NULL);
  result->Clear(0);

  list<unsigned int> MURs = config.listMURSInRod(partition, crate, rod);

  for(list<unsigned int> :: const_iterator m = MURs.begin();
      m!=MURs.end();
      m++) {
    result->Append(*m);
  }

  return result;
}

sctConf_T_ModuleList ConfigServer::listModulesInMUR (sctConfStatus *_status, ilu_Cardinal partition, ilu_Cardinal MUR) 
{
  sctConf_T_ModuleList result;

  result = _sctConf_T_ModuleList_sequence::Create(4, NULL);
  result->Clear(0);

  list<string> mods = config.listModulesInMUR(partition, MUR);

  for(list<string> :: const_iterator m = mods.begin();
      m!=mods.end();
      m++) {
    char *copy = (char *)malloc((m->length()+1) * sizeof(char));
    strncpy(copy, m->c_str(), m->length());
    copy[m->length()] = 0;
    result->Append(copy);
  }

  return result;
}

sctConf_T_ModuleList ConfigServer::listRModulesInMUR (sctConfStatus *_status, ilu_Cardinal partition, ilu_Cardinal MUR) 
{
  sctConf_T_ModuleList result;

  result = _sctConf_T_ModuleList_sequence::Create(4, NULL);
  result->Clear(0);

  list<string> mods = config.listRModulesInMUR(partition, MUR);

  for(list<string> :: const_iterator m = mods.begin();
      m!=mods.end();
      m++) {
    char *copy = (char *)malloc((m->length()+1) * sizeof(char));
    strncpy(copy, m->c_str(), m->length());
    copy[m->length()] = 0;
    result->Append(copy);
  }

  return result;
}

sctConf_T_ModuleList ConfigServer::listAllModules (sctConfStatus *_status) 
{
  sctConf_T_ModuleList result = 0;

  try {
    result = _sctConf_T_ModuleList_sequence::Create(4, NULL);
    result->Clear(0);

    list<string> mods = config.listAllModules();

    for(list<string> :: const_iterator m = mods.begin();
        m!=mods.end();
        m++) {
      char *copy = (char *)malloc((m->length()+1) * sizeof(char));
      strncpy(copy, m->c_str(), m->length());
      copy[m->length()] = 0;
      result->Append(copy);
    }
  } catch(ConfigurationException &c) {
    myRaise(_status, c);
  }

  return result;
}

sctConf_T_ModuleList ConfigServer::listUnusedModules (sctConfStatus *_status) 
{
  sctConf_T_ModuleList result = 0;
  try {
    result = _sctConf_T_ModuleList_sequence::Create(4, NULL);
    result->Clear(0);

    list<string> mods = config.listUnusedModules();

    for(list<string> :: const_iterator m = mods.begin();
        m!=mods.end();
        m++) {
      char *copy = (char *)malloc((m->length()+1) * sizeof(char));
      strncpy(copy, m->c_str(), m->length());
      copy[m->length()] = 0;
      result->Append(copy);
    }
  } catch(ConfigurationException &c) {
    myRaise(_status, c);
  }

  return result;
}

sctConf_T_ABCDModule* ConfigServer::getModuleConfig (sctConfStatus *_status, ilu_T_CString mod) 
{
  try {
    ABCDModule realConfig = config.getModuleConfig(mod);
    return copyABCDModuleToILU(realConfig);

  } catch(ConfigurationException &c) {
    myRaise(_status, c);
    return 0;
  }
}

sctConf_T_RodConfig * ConfigServer::getRodConfig (sctConfStatus *_status, ilu_Cardinal partition, ilu_Cardinal crate, ilu_Cardinal rod) 
{
  sctConf_T_RodConfig *result = 0;

  try {
    result = new sctConf_T_RodConfig;

    RodConfig ret = config.getRodConfig(partition, crate, rod);

    result->baseAddress = ret.baseAddress;
    result->mapSize = ret.mapSize;
    result->numSlaves = ret.numSlaves;

    result->slaves = _sctConf_T_SlaveList_sequence::Create(4, NULL);
    result->slaves->Clear(0);

    for(int i=0; i<4; i++) {
      sctConf_T_SlaveConfig *slave = new sctConf_T_SlaveConfig;

      SlaveConfig &conf = ret.slaves[i];

      slave->emifFile = (char *)malloc((conf.emifFile.length()+1) * sizeof(char));
      strncpy(slave->emifFile, conf.emifFile.c_str(), conf.emifFile.length());
      slave->emifFile[conf.emifFile.length()] = 0;

      slave->ipramFile = (char *)malloc((conf.ipramFile.length()+1) * sizeof(char));
      strncpy(slave->ipramFile, conf.ipramFile.c_str(), conf.ipramFile.length());
      slave->ipramFile[conf.ipramFile.length()] = 0;

      slave->idramFile = (char *)malloc((conf.idramFile.length()+1) * sizeof(char));
      strncpy(slave->idramFile, conf.idramFile.c_str(), conf.idramFile.length());
      slave->idramFile[conf.idramFile.length()] = 0;

      slave->extFile = (char *)malloc((conf.extFile.length()+1) * sizeof(char));
      strncpy(slave->extFile, conf.extFile.c_str(), conf.extFile.length());
      slave->extFile[conf.extFile.length()] = 0;

      result->slaves->Append(slave);
    }
  } catch(ConfigurationException &c) {
    myRaise(_status, c);
  }

// #warning "Copy ROD Config"
//   cout << "Can't copy ROD config\n";

  return result;
}

sctConf_T_TimConfig * ConfigServer::getTimConfig (sctConfStatus *_status, ilu_Cardinal partition, ilu_Cardinal crate) 
{
  TimConfig myTim = config.getTimConfig(partition, crate);

  return copyTimConfigToILU(myTim);
}

sctConf_T_CharBlock ConfigServer::getFibreMappings (sctConfStatus *_status, ilu_Cardinal partition, ilu_Cardinal crate, ilu_Cardinal rod)
{
  char * mappings = 0;
  try {
    mappings = config.getFibreMappings(partition, crate, rod);
  } catch(ConfigurationException &c) {
    myRaise(_status, c);
  }
  return copyArrayToILU<_sctConf_T_CharBlock_sequence, char>(mappings, 48*3);
}

void ConfigServer::setFibreMapping (sctConfStatus *_status, 
                                    ilu_Cardinal partition, ilu_Cardinal crate, ilu_Cardinal rod, 
                                    ilu_Cardinal channel, ilu_Cardinal tx, ilu_Cardinal rx0, ilu_Cardinal rx1)
{
  try {
    config.setFibreMapping(partition, crate, rod, channel, tx, rx0, rx1);
  } catch(ConfigurationException &c) {
    myRaise(_status, c);
  }
}

sctConf_T_MURType ConfigServer::getMURType (sctConfStatus *_status, ilu_Cardinal MUR) 
{
  MURType result = UNKNOWN;
  try {
    result = config.getMURType(MUR);
  } catch(ConfigurationException &c) {
    myRaise(_status, c);
  }

  return (sctConf_T_MURType)result;
}

void ConfigServer::translateToROD (sctConfStatus *_status, ilu_Cardinal MUR, ilu_Cardinal mod, 
                                   ilu_Cardinal * partition, ilu_Cardinal * crate, ilu_Cardinal * rod, ilu_Cardinal * channel) 
{
  try {
    unsigned int partitionOut, crateOut, rodOut, channelOut;

    config.translateToROD(MUR, mod, partitionOut, crateOut, rodOut, channelOut);

    *partition = partitionOut;
    *crate = crateOut;
    *rod = rodOut;
    *channel = channelOut;
  } catch(ConfigurationException &c) {
    myRaise(_status, c);
  }
}

void ConfigServer::translateToRROD (sctConfStatus *_status, ilu_Cardinal MUR, ilu_Cardinal mod, 
                                   ilu_Cardinal * partition, ilu_Cardinal * crate, ilu_Cardinal * rod, ilu_Cardinal * channel) 
{
  try {
    unsigned int partitionOut, crateOut, rodOut, channelOut;

    config.translateToRROD(MUR, mod, partitionOut, crateOut, rodOut, channelOut);

    *partition = partitionOut;
    *crate = crateOut;
    *rod = rodOut;
    *channel = channelOut;
  } catch(ConfigurationException &c) {
    myRaise(_status, c);
  }
}

void ConfigServer::translateToSN (sctConfStatus *_status, ilu_Cardinal MUR, ilu_Cardinal mod, 
                                  ilu_T_CString * sn) 
{
  try {
    std::string serialNum;

    config.translateToSN(MUR, mod, serialNum);

    *sn = new char[serialNum.length()+1];

    memcpy(*sn, serialNum.c_str(), serialNum.length());
    (*sn)[serialNum.length()] = 0;
  } catch(ConfigurationException &c) {
    myRaise(_status, c);
  }
}

sctConf_T_BOCChannelConfig * ConfigServer::getBOCConfig (sctConfStatus *_status, ilu_Cardinal partition, ilu_Cardinal crate, ilu_Cardinal rod, ilu_Cardinal channel) 
{
  sctConf_T_BOCChannelConfig *res = 0;
  try {
    res = new sctConf_T_BOCChannelConfig;

    BOCChannelConfig ret = config.getBOCConfig(partition, crate, rod, channel);

    res->current = ret.current;
    res->delay = ret.delay;
    res->markSpace = ret.markSpace;

    res->threshold0 = ret.threshold0;
    res->delay0 = ret.delay0;

    res->threshold1 = ret.threshold1;
    res->delay1 = ret.delay1;
  } catch(ConfigurationException &c) {
    myRaise(_status, c);
  }

  return res;
}

void ConfigServer::printModuleConfig (sctConfStatus *_status, sctConf_T_ABCDModule* conf) 
{
  try {
    config.printModuleConfig(copyILUToABCDModule(*conf));
  } catch(ConfigurationException &c) {
    myRaise(_status, c);
  }
}

void ConfigServer::configureBOCChannel (sctConfStatus *_status, ilu_Cardinal MUR, ilu_Cardinal position, sctConf_T_BOCChannelConfig * conf) 
{
  try {
    BOCChannelConfig newConf;

    newConf.current = conf->current;
    newConf.delay = conf->delay;
    newConf.markSpace = conf->markSpace;

    newConf.threshold0 = conf->threshold0;
    newConf.delay0 = conf->delay0;

    newConf.threshold1 = conf->threshold1;
    newConf.delay1 = conf->delay1;

    config.configureBOCChannel(MUR, position, newConf); 
  } catch(ConfigurationException &c) {
    myRaise(_status, c);
  }
}


// void ConfigServer::setBOCConfig (sctConfStatus *_status, ilu_Cardinal partition, ilu_Cardinal crate, ilu_Cardinal rod, ilu_Cardinal channel, sctConf_T_BOCChannelConfig * conf) 
// {
//   try {
//     BOCChannelConfig newConf;

//     newConf.current = conf->current;
//     newConf.delay = conf->delay;
//     newConf.markSpace = conf->markSpace;

//     newConf.threshold0 = conf->threshold0;
//     newConf.delay0 = conf->delay0;

//     newConf.threshold1 = conf->threshold1;
//     newConf.delay1 = conf->delay1;

//     config.setBOCConfig(partition, crate, rod, channel, newConf); 
//   } catch(ConfigurationException &c) {
//     myRaise(_status, c);
//   }
// }

void ConfigServer::translateToBarrel (sctConfStatus *_status, ilu_Cardinal MUR, ilu_Cardinal mod, 
                                      ilu_Cardinal * barrel, ilu_Cardinal * row, ilu_Integer * number) 
{
  try {
    unsigned int b, r;
    int num;

    config.translateToBarrel(MUR, mod, b, r, num);

    *barrel = b;
    *row = r;
    *number = num;
  } catch(ConfigurationException &c) {
    myRaise(_status, c);
  }
}

void ConfigServer::translateToEndcap (sctConfStatus *_status, ilu_Cardinal MUR, ilu_Cardinal mod, 
                                      ilu_Cardinal * disk, ilu_Cardinal * ring, ilu_Cardinal * number) 
{
  try {
    unsigned int d, r, n;
    config.translateToEndcap(MUR, mod, d, r, n);
    *disk = d;
    *ring = r;
    *number = n;
  } catch(ConfigurationException &c) {
    myRaise(_status, c);
  }
}

void ConfigServer::translateToPowerSupply (sctConfStatus *_status, ilu_Cardinal MUR, ilu_Cardinal mod, ilu_Cardinal * partition, ilu_Cardinal * crate, ilu_Cardinal * channel) 
{
  try {
    unsigned int partitionOut, crateOut, channelOut;

    config.translateToPowerSupply(MUR, mod, partitionOut, crateOut, channelOut);

    *partition = partitionOut;
    *crate = crateOut;
    *channel = channelOut;
  } catch(ConfigurationException &c) {
    myRaise(_status, c);
  }
}

void ConfigServer::translateFromROD (sctConfStatus *_status, ilu_Cardinal partition, ilu_Cardinal crate, ilu_Cardinal rod, ilu_Cardinal channel, 
                                     ilu_Cardinal * MUR, ilu_Cardinal * mod) 
{
  try {
    unsigned int M, module;
    config.translateFromROD(partition, crate, rod, channel, M, module);
    *MUR = M;
    *mod = module;
  } catch(ConfigurationException &c) {
    myRaise(_status, c);
  }
}

void ConfigServer::translateFromRROD (sctConfStatus *_status, ilu_Cardinal partition, ilu_Cardinal crate, ilu_Cardinal rod, ilu_Cardinal channel, 
                                     ilu_Cardinal * MUR, ilu_Cardinal * mod) 
{
  try {
    unsigned int M, module;
    config.translateFromRROD(partition, crate, rod, channel, M, module);
    *MUR = M;
    *mod = module;
  } catch(ConfigurationException &c) {
    myRaise(_status, c);
  }
}

void ConfigServer::translateFromSN (sctConfStatus *_status, ilu_T_CString sn, 
                                    ilu_Cardinal * MUR, ilu_Cardinal * mod) 
{
  try {
    unsigned int M, module;
    config.translateFromSN(std::string(sn), M, module);
    *MUR = M;
    *mod = module;
  } catch(ConfigurationException &c) {
    myRaise(_status, c);
  }
}

void ConfigServer::translateFromBarrel (sctConfStatus *_status, ilu_Cardinal barrel, ilu_Cardinal row, ilu_Integer number, 
                                        ilu_Cardinal * MUR, ilu_Cardinal * mod) 
{
  try {
    unsigned int M, module;
    config.translateFromBarrel(barrel, row, number, M, module);
    *MUR = M;
    *mod = module;
  } catch(ConfigurationException &c) {
    myRaise(_status, c);
  }
}

void ConfigServer::translateFromEndcap (sctConfStatus *_status, ilu_Cardinal disk, ilu_Cardinal ring, ilu_Cardinal number, 
                                        ilu_Cardinal * MUR, ilu_Cardinal * mod) 
{
  try {
    unsigned int M, module;
    config.translateFromEndcap(disk, ring, number, M, module);
    *MUR = M;
    *mod = module;
  } catch(ConfigurationException &c) {
    myRaise(_status, c);
  }
}

void ConfigServer::translateFromPowerSupply (sctConfStatus *_status, ilu_Cardinal partition, ilu_Cardinal crate, ilu_Cardinal channel, ilu_Cardinal * MUR, ilu_Cardinal * mod) 
{
  try {
    unsigned int M, module;
    config.translateFromPowerSupply(partition, crate, channel, M, module);
    *MUR = M;
    *mod = module;
  } catch(ConfigurationException &c) {
    myRaise(_status, c);
  }
}

void ConfigServer::configureModuleFromStructure (sctConfStatus *_status, ilu_T_CString mod, sctConf_T_ABCDModule* conf)
{
  ABCDModule modConfig = copyILUToABCDModule(*conf);

  try {
    config.configureModuleFromStructure(mod, modConfig);
//     cout << "Can't update config\n";
//     throw ConfigurationException("Can't update config\n");
  } catch(ConfigurationException &e) {
    myRaise(_status, e);
  }
}

void ConfigServer::configureModuleFromFile (sctConfStatus *_status, ilu_T_CString filename)
{
  try {
    config.configureModuleFromFile(filename);
  } catch(ConfigurationException &e) {
    myRaise(_status, e);
  }
}

void ConfigServer::clearAll (sctConfStatus *_status)
{
  try {
    config.clearAll();
  } catch(ConfigurationException &e) {
    myRaise(_status, e);
  }
}

void ConfigServer::namePartition (sctConfStatus *_status, ilu_Cardinal partition, ilu_T_CString name)
{
  try {
    config.namePartition(partition, name);
  } catch(ConfigurationException &e) {
    myRaise(_status, e);
  }
}

// void ConfigServer::configureROD (sctConfStatus *_status, 
//                                  ilu_Cardinal partition, ilu_Cardinal crate, ilu_Cardinal rod, 
//                                  sctConf_T_RodConfig * conf) 
// {
//   cout << "Can't update ROD config\n";
//   //  config.updateRodConfig(partition, crate, rod, conf);
// }

void ConfigServer::loadConfiguration (sctConfStatus *_status, ilu_T_CString filename) 
{
  try {
    config.loadConfiguration(filename);
  } catch(ConfigurationException &c) {
    myRaise(_status, c);
  }
}

void ConfigServer::saveConfiguration (sctConfStatus *_status, ilu_T_CString filename) 
{
  try {
    config.saveConfiguration(filename);
  } catch(ConfigurationException &c) {
    myRaise(_status, c);
  }
}

void ConfigServer::saveModuleConfiguration (sctConfStatus *_status, ilu_T_CString sn, ilu_T_CString filename) 
{
  try {
    config.saveModuleConfiguration(sn, filename);
  } catch(ConfigurationException &c) {
    myRaise(_status, c);
  }
}

ilu_T_CString ConfigServer::getModuleConfigurationString (sctConfStatus *_status, ilu_T_CString modsn, sctConf_T_ABCDModule* mConf)
{
  ilu_T_CString result = 0;

  try {
    string buffer = config.getModuleConfigurationString(modsn, copyILUToABCDModule(*mConf));

    result = new char[buffer.length() + 1];

    strncpy(result, buffer.c_str(), buffer.length() + 1);
  } catch(ConfigurationException &c) {
    myRaise(_status, c);
  }

  return result;
}

void ConfigServer::writePowerSupplyConfiguration (sctConfStatus *_status, ilu_T_CString filename)
{
  try {
    config.writePowerSupplyConfiguration(filename);
  } catch(ConfigurationException &c) {
    myRaise(_status, c);
  }
}

void ConfigServer::mapRODMUR (sctConfStatus *_status, 
                              ilu_Cardinal partition, ilu_Cardinal crate, ilu_Cardinal rod, 
                              ilu_Cardinal order, ilu_Cardinal number) 
{ 
  try {
    config.mapRODMUR(partition, crate, rod, order, number);
  } catch(ConfigurationException &c) {
    myRaise(_status, c);
  }
}

void ConfigServer::mapBarrelMUR (sctConfStatus *_status, 
                                 ilu_Cardinal MUR, 
                                 ilu_Cardinal barrel, ilu_Cardinal row, ilu_Cardinal position)
{
  try {
    config.mapBarrelMUR(MUR, barrel, row, position);
  } catch(ConfigurationException &c) {
    myRaise(_status, c);
  }
}

void ConfigServer::mapEndcapMUR (sctConfStatus *_status, 
                                 ilu_Cardinal MUR, 
                                 ilu_Cardinal disk, ilu_Cardinal quadrant, ilu_Cardinal position)
{
  try {
    config.mapEndcapMUR(MUR, disk, quadrant, position);
  } catch(ConfigurationException &c) {
    myRaise(_status, c);
  }
}

void ConfigServer::mapPowerChannel (sctConfStatus *_status, ilu_Cardinal MUR, ilu_Cardinal number, ilu_Cardinal partition, ilu_Cardinal crate, ilu_Cardinal channel)
{
  try {
    config.mapPowerChannel(MUR, number, partition, crate, channel);
  } catch(ConfigurationException &c) {
    myRaise(_status, c);
  }
}

void ConfigServer::mapModuleMUR (sctConfStatus *_status, 
                                 ilu_Cardinal MUR, ilu_Cardinal order, ilu_Cardinal RMUR, ilu_Cardinal rorder, 
                                 ilu_T_CString number) 
{
  try {
    config.mapModuleMUR(MUR, order, RMUR, rorder, number);
  } catch(ConfigurationException &c) {
    myRaise(_status, c);
  }
}

void ConfigServer::swapMURNames (sctConfStatus *_status, 
                                 ilu_Cardinal MUR1, ilu_Cardinal MUR2) 
{
  try {
    config.swapMURNames(MUR1, MUR2);
  } catch(ConfigurationException &c) {
    myRaise(_status, c);
  }
}

void ConfigServer::modifyPowerParam (sctConfStatus *_status, 
                                     ilu_Cardinal MUR, ilu_Cardinal number, ilu_T_CString name, ilu_ShortReal value) 
{
  try {
    config.modifyPowerParam(MUR, number, name, value);
  } catch(ConfigurationException &c) {
    myRaise(_status, c);
  }
}

void ConfigServer::modifyDefaultPowerParam (sctConfStatus *_status, 
                                            ilu_T_CString name, ilu_ShortReal value) 
{
  try {
    config.modifyDefaultPowerParam(name, value);
  } catch(ConfigurationException &c) {
    myRaise(_status, c);
  }
}

ilu_ShortReal ConfigServer::getPowerParam (sctConfStatus *_status, 
                                           ilu_Cardinal MUR, ilu_Cardinal number, ilu_T_CString name) 
{
  ilu_ShortReal result;

  try {
    result = config.getPowerParam(MUR, number, name);
  } catch(ConfigurationException &c) {
    myRaise(_status, c);
  }

  return result;
}

void ConfigServer::configureROD (sctConfStatus *_status, 
                                 ilu_Cardinal partition, ilu_Cardinal crate, ilu_Cardinal rod, 
                                 sctConf_T_RodConfig * conf) 
{
  try {
    config.configureROD(partition, crate, rod, copyILUToRodConfig(conf));
  } catch(ConfigurationException &c) {
    myRaise(_status, c);
  }
}

void ConfigServer::configureTIM (sctConfStatus *_status, 
                                 ilu_Cardinal partition, ilu_Cardinal crate, 
                                 sctConf_T_TimConfig * conf) 
{
  try {
    config.configureTIM(partition, crate, copyILUToTimConfig(conf));
  } catch(ConfigurationException &c) {
    myRaise(_status, c);
  }
}


int main(int argc, char** argv) 
{
  IPCCore::init(false);
  IPCPartition p("SCT");
  IPCServer* server = new IPCServer("Test_Server", p);

  ConfigServer *h = new ConfigServer("SCTConfigurationServer", server);
  if (!h->publish()) {
    cout << "Failed to publish Configuration" << endl;
    return 0;
  } 
  cout << "Configuration server published, awaiting commands\n";
  try {
    server->run();
  } catch(ConfigurationException &c) {
    cout << "Configuration run threw exception " << c.what() << endl;
  }
   
  cout << "Run over" << endl;

  h->withdraw();
  delete h;
  delete server;
}

void myRaise(sctConfStatus *_status, ConfigurationException &c) 
{
  ConfException *e = new ConfException;
  char *buffer = new char[strlen(c.what())+1];

  cout << "Raising exception " << c.what() << endl;

  strcpy(buffer, c.what());
  buffer[strlen(c.what())] = 0;
  e->detail = buffer;
  sctConf_G::RaiseException(_status, sctConf_E_ConfigurationException, e);
}
