package DisplayGUI;

import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.text.*;
import java.util.regex.*;
import is.*;
import Sct.Serializable;
import Sct.IS.*;
import Sct_CalibrationController.TestData;


/*
 * SCTDBDataInterface.java
 *
 */

/**
 *
 * @author  robinson
 */
public class SCTDBDataInterface implements TestDataInfo, InfoListener {
    
    gui guiControl;
    Hashtable serverHash;  // hash to keep track of which servers we are subscribed to...
    String theObjectName;
    Vector ISTests;
    int testIndex;            // the currently selected test
    int listFilterIndex;      // the requested type of test
    TestData testData;
    String testLocation;

    Pattern objectPattern;

//    String resultObjectRegex;
    String controlPattern;
    String resultObjectHeader;   // header of the current selected result object name

    String SCTDBServer = "SCTDBData";

    public SCTDBDataInterface(gui parent) {
        guiControl=parent;
//        sct = SCTInterface2.getInstance();
        is = GuiComponents.System.ISInterface.getInstance();
        serverHash = new Hashtable();
        ISTests = new Vector();
//        resultObjectRegex = "(.*)Summary\\.([0-9]+)\\.([0-9]+)(\\.[0-9]{14}|\\.SCTTestAPI_PseudoModule)?"; // look for the runno and startscanno
//        objectPattern = Pattern.compile(resultObjectRegex);
        controlPattern = "(.*)TestData\\.([0-9])+\\.([0-9])+";
        objectPattern = Pattern.compile(controlPattern);
        testIndex=-1;
        listFilterIndex=-1;
        testLocation="Any";
    }
    


    

    protected void subscribe(int testDataType) {
// subscribe to control server to keep up to date with the list of tests
        listFilterIndex=testDataType;
        try {

//         String testServer = SctNames.getTestISServer();
         String controlServer = SctNames.getControlISServer();
         if(!serverHash.containsKey(controlServer)) {
             is.subscribe(controlServer, controlPattern, true, this);
//             System.out.println("SCTDBDataInterface: Subscribed to "+controlServer+" for objects matching "+controlPattern);
             serverHash.put(controlServer, controlPattern);
             }
         if(!serverHash.containsKey(SCTDBServer)) {
             is.subscribe(SCTDBServer, ".*", true, this);
//             System.out.println("SCTDBDataInterface: Subscribed to "+SCTDBServer+" for objects matching .*");
             serverHash.put(SCTDBServer, ".*");
             }
         String[] objects = is.getISObjects(controlServer,controlPattern);
         ISTests = new Vector();
// require a the result object to have a matching control object
         for (int j=0; j<objects.length; j++) {  
//                  System.out.println(objects[j]);
                  Matcher matcher = objectPattern.matcher(objects[j]);
                  if(!matcher.matches()) continue;
//                  String controlObject = "ControlData.TestData."+objects[j].substring(matcher.start(2),matcher.end(2))+"."+objects[j].substring(matcher.start(3),matcher.end(3));
                  String controlObject = objects[j];
                  if(listFilterIndex==-1) ISTests.addElement(controlObject);
                   else {
                       TestData testData = new TestData(); 
                       is.getRepository().getValue(controlObject, testData); 
                       if(testData.testName.indexOf(testNames[listFilterIndex])!=-1) ISTests.addElement(controlObject);
                       }
                   }  
        } catch (RuntimeException e) {
            System.err.println("Couldn't subscribe to one or more IS Server(s): " + e.getMessage());
        }
//        guiControl.displayPane.setDisplayColors();
//        guiControl.displayPane.refreshDisplay();   
    }

    protected void unsubscribe() {
            for(Enumeration e = serverHash.keys(); e.hasMoreElements();) {
               String server = (String)e.nextElement();
               String pattern = (String)serverHash.get(server);
               is.unsubscribe(server, pattern);
//               System.out.println(is.unsubscribe(server, pattern));
               }
            serverHash = new Hashtable();
            testIndex=-1;
            listFilterIndex=-1;
    }


    
    /** InfoListener interfaces
     */ 
    
    /**
     * Adds the new object - we must run this in the EventDispatch thread to avoid possible thread conflicts
     */
    public void infoCreated(InfoEvent e) {
       theObjectName = e.getName();
       javax.swing.SwingUtilities.invokeLater(new Runnable() {
          public void run() {
                  Matcher matcher = objectPattern.matcher(theObjectName);
                  if(matcher.matches()) { // new result
                    String controlObject = "ControlData.TestData."+theObjectName.substring(matcher.start(2),matcher.end(2))+"."+theObjectName.substring(matcher.start(3),matcher.end(3)); 
                    if(!ISTests.contains(controlObject)) ISTests.addElement(controlObject);
                    }
                  guiControl.tablesDisplayPane.updateTables(true);  // immediately add the control info to the table
                  guiControl.displayPane.setDisplayColors();
                  guiControl.displayPane.refreshDisplay();  // just refresh the display
          }
        });
    }


    
    /**
     * Object removed from IS -must run this in the EventDispatch thread to avoid possible conflicts
     */
    public void infoDeleted(InfoEvent e) {
       theObjectName = e.getName(); 
       javax.swing.SwingUtilities.invokeLater(new Runnable() {
          public void run() {   
             if(ISTests.contains(theObjectName)) {
                 ISTests.removeElement(theObjectName);
                 }
             guiControl.displayPane.setDisplayColors();
             guiControl.displayPane.refreshDisplay();  // just refresh the display
          }
        });
    }
    
    /**
     * Does nothing - object is still there 
     */
    public void infoUpdated(InfoEvent e) {
        theObjectName = e.getName();
        if(theObjectName.indexOf("ControlData")!=-1) guiControl.tablesDisplayPane.updateTestTable(theObjectName,ISTests.indexOf(theObjectName));
//        System.out.println(e.getName() + " has been updated");
    }
//***************************************************************************
// construct a tabel vector of the current tests
   public Vector getTestTableList() {
       Vector theLine = new Vector();
       Vector tableContent = new Vector();
       theLine.addElement("Test");
       theLine.addElement("Status");
       theLine.addElement("Run");
       theLine.addElement("Scan");
       theLine.addElement("#Scans");
       tableContent.addElement(theLine);
       for(int i=0;i<ISTests.size();i++) {
           TestData testData = new TestData(); 
           is.getRepository().getValue((String)ISTests.elementAt(i), testData); 
           theLine=new Vector();
           if(testData.testName.equals("NPtGainTest") && testData.nScans==3) testData.testName = "3PtGainTest";
           theLine.addElement(testData.testName);
           switch(testData.status) {
             case 0:
                 theLine.addElement("Executing");
                 break;
             case 1:
                 theLine.addElement("Completed");
                 break;
             case 2:
                 theLine.addElement("Aborted");
                 break;
             default:
                 theLine.addElement("Unknown");
             }
           theLine.addElement(Integer.toString(testData.runNumber));
           theLine.addElement(Integer.toString(testData.startScanNumber));
           theLine.addElement(Integer.toString(testData.nScans));
           tableContent.addElement(theLine);
           }
       return tableContent;
    }

        
//****************************************************************************
     public void setTestFilter(int selectedRow) {
         if(selectedRow==-1) {
             resultObjectHeader=null;
            testIndex=-1;
            return;
            }
         String testObjectName = (String)ISTests.elementAt(selectedRow);
         testData = new TestData(); 
         is.getRepository().getValue(testObjectName, testData); 
// first extract the run number, and scan numbers
         String testName = testData.testName;
         testIndex=-1;
         for(int i=0;i<testNames.length;i++) {
           if(testName.indexOf(testNames[i])!=-1) {
                    if(testName.equals("NPtGainTest") && testData.nScans==3) {
                             testIndex=5;
                             break;
                             }
                    testIndex=i;
                    break;
                    }         
           }
          resultObjectHeader = "SCTDBData."+testNames[testIndex]+"."+testLocation+".";
         } 
//***********************************************************************8
     public String getResultObjectHeader() {
          return resultObjectHeader;
          }
//**********************************************************************8
     public int getTestIndex() {
         return testIndex;
         }
//*********************************************************************
     public int getListFilterIndex() {
         return listFilterIndex;
         }
   public int getRunNo() {
         if(testData==null) return -1;
         else return testData.runNumber;
         }
   public int getScanNo() {
         if(testData==null) return -1;
         else return testData.startScanNumber;
         }
   public void downloadTests(int index) {
         boolean refresh=false;
         if(index==-1) {
           index=testIndex;
           refresh=true;
           }
         if(testIndex==-1) return;
         SCTDBTestDownloader d = new SCTDBTestDownloader(guiControl.config.getSortedVector(guiControl.displayPane.getSCTView()),testIndex,testLocation,true,refresh);
         if(d.downloadsRequired() && d.isOkToStart()) d.start();
         }
   public void setLocation(String location) {
        testLocation=location;
        }
   public String getLocation() {
        return testLocation;
        }


     private GuiComponents.System.ISInterface is;

}
