package DisplayGUI;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

import java.util.*;

//SelectorPane itself is not a visible component.
public class SelectorPane extends JPanel implements ListSelectionListener,displayParams,SCTInfo,TestDataInfo {
    private JList displayParameters, sctViews, mouseOptions;
    private JSplitPane splitPane;
    private String[] defaultString = {"...None"};
    private String[][] displayParameterStrings = {
                          {},
                          {"Module Group"},
                          {"Raw Data","Fitted Data","Analysed Data"}
                       };
    private String[][] mouseOptionStrings = {
                         {},
                         {"Add to Group 1","Add to Group 2"},
                         {"List Data objects"},
                         {},
                         {}
                       };
    private gui guiControl;

    private boolean selectionListenerIsActive=true;

    public SelectorPane(gui parent) {
        guiControl=parent;
        sctViews = new JList(SCTViewNames);
        sctViews.setFont(displayScaleFont);
        sctViews.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        sctViews.setSelectedIndex(0);
        sctViews.addListSelectionListener(this);
        JScrollPane viewScrollPane = new JScrollPane(sctViews);
        JLabel header2 = new JLabel("Views");
        header2.setForeground(darkBlueColor);
        header2.setBackground(backgroundColor);
        header2.setFont(displayTextFont);
        viewScrollPane.setColumnHeaderView(header2);
        viewScrollPane.getColumnHeader().setBackground(backgroundColor);
        viewScrollPane.setBorder(new javax.swing.border.EmptyBorder(0,0,0,0));
        viewScrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        viewScrollPane.getVerticalScrollBar().setBackground(backgroundColor);

        displayParameters = new JList();
        displayParameters.setFont(displayScaleFont);
        displayParameters.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

        displayParameters.addListSelectionListener(this);
        JScrollPane paramScrollPane = new JScrollPane(displayParameters);
        JLabel header1 = new JLabel("Parameters");
        header1.setForeground(darkBlueColor);
        header1.setBackground(backgroundColor);
        header1.setFont(displayTextFont);
        paramScrollPane.setColumnHeaderView(header1);
        paramScrollPane.getColumnHeader().setBackground(backgroundColor);
        paramScrollPane.setBorder(new javax.swing.border.EmptyBorder(0,0,0,0));
//        paramScrollPane.setBorder(BorderFactory.createLineBorder(Color.black));
        paramScrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        paramScrollPane.getVerticalScrollBar().setBackground(backgroundColor);

        mouseOptions = new JList();
        mouseOptions.setFont(displayScaleFont);
        mouseOptions.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        mouseOptions.addListSelectionListener(this);
        JScrollPane mouseScrollPane = new JScrollPane(mouseOptions);
        JLabel header3 = new JLabel("Options");
        header3.setForeground(darkBlueColor);
        header3.setBackground(backgroundColor);
        header3.setFont(displayTextFont);
        mouseScrollPane.setColumnHeaderView(header3);
        mouseScrollPane.getColumnHeader().setBackground(backgroundColor);
        mouseScrollPane.setBorder(new javax.swing.border.EmptyBorder(0,0,0,0));
        mouseScrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        mouseScrollPane.getVerticalScrollBar().setBackground(backgroundColor);

        //Create a split pane with the two scroll panes in it
        JSplitPane topSplit = new JSplitPane(JSplitPane.VERTICAL_SPLIT, paramScrollPane, mouseScrollPane);
        topSplit.setDividerSize(8);
        topSplit.setDividerLocation(0.50);
        topSplit.setBorder(new javax.swing.border.EmptyBorder(0,0,0,0));

        splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
                                   topSplit, viewScrollPane);
//        splitPane.setOneTouchExpandable(true);
        splitPane.setDividerLocation(0.65);
        splitPane.setDividerSize(8);
        splitPane.setBorder(new javax.swing.border.EmptyBorder(0,0,0,0));

        //Provide minimum sizes for the two components in the split pane
        Dimension minimumSize = new Dimension(95, 16);
        paramScrollPane.setMinimumSize(minimumSize);
        mouseScrollPane.setMinimumSize(minimumSize);
        viewScrollPane.setMinimumSize(minimumSize);

        //Provide a preferred size for the split pane
        splitPane.setPreferredSize(new Dimension(95,yOffSet+56*boxHeight)); 
        add(splitPane);
        setBackground(backgroundColor);
        setBorder(new javax.swing.border.TitledBorder(null, "Display", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("Arial", 0, 11), new java.awt.Color(0, 0, 153)));
//        setContentPane(splitPane);

//        pack();
//        setVisible(true);

    }
    public void initParamList() {

        int displayType = guiControl.displayStatus.getStatus();
        boolean noParameters=true, noOptions=true;
        int testIndex = -1;
        if(displayType==DisplayStatus.UNDEFINED) return;
        switch(displayType) {
            case DisplayStatus.MODULE_GROUP_SELECTION:
            case DisplayStatus.IS_MONITOR:
               displayParameters.setListData(displayParameterStrings[displayType]);
               mouseOptions.setListData(mouseOptionStrings[displayType]);
               noParameters=false;
               noOptions=false;
               break;
            case DisplayStatus.TEST_DATA:
               if((testIndex=guiControl.testDataInterface.getTestIndex())!=-1) {
                           displayParameters.setListData(testParameters[testIndex]);
                           noParameters=false;
                           }
               mouseOptions.setListData(chipOptions);
               noOptions=false;
               break;
            case DisplayStatus.SCTDB_DATA:
               if((testIndex=guiControl.sctdbDataInterface.getTestIndex())!=-1) {
                          displayParameters.setListData(testParameters[testIndex]);
                          noParameters=false;
                          }
               mouseOptions.setListData(chipOptions);
               noOptions=false;
               break;
            case DisplayStatus.UNDEFINED:
            default:
              displayParameters = new JList();
              mouseOptions = new JList();
            }


        if(displayParameters.getModel().getSize()>0) {
           selectionListenerIsActive=false;
           displayParameters.setSelectedIndex(0);
           }
        if(mouseOptions.getModel().getSize()>0) {
            selectionListenerIsActive=false;
            mouseOptions.setSelectedIndex(0);
            }
        if(noParameters) displayParameters.setListData(defaultString);
        if(noOptions) mouseOptions.setListData(defaultString);
        setVisible(true);
    }

   public int getDisplayParameterIndex() {
     return displayParameters.getSelectedIndex();
     }
   public int getMouseClickIndex() {
     return mouseOptions.getSelectedIndex();
     }
   public String getOption() {
      return (String)mouseOptions.getSelectedValue();
      }
   public String getDisplayParameter() {
     return (String)displayParameters.getSelectedValue();
     }

    public JSplitPane getSplitPane() {
        return splitPane;
    }

    public void valueChanged(ListSelectionEvent e) {

        if (e.getValueIsAdjusting())
            return;

        JList theList = (JList)e.getSource();
        if (theList.isSelectionEmpty()) {
              // huh?
        } else {
        if(!selectionListenerIsActive) {
             selectionListenerIsActive=true;
             return;
             }
            int index = theList.getSelectedIndex();
            if(theList==sctViews) {
                   guiControl.displayPane.setSCTView(index);
                   return;
                   }
            switch(guiControl.displayStatus.getStatus()) {
            case DisplayStatus.IS_MONITOR:
              guiControl.isInterface.setServerName(index);
              guiControl.colorScalePane.setColorScale();  
//              System.out.println("SelectorPane::ISmonitor option setDisplayColors");
              guiControl.displayPane.setDisplayColors();    
              guiControl.displayPane.refreshDisplay();
              ModuleCell lastClickedCell;
              if((lastClickedCell=guiControl.displayPane.getLastClickedCell())!=null) guiControl.tablesDisplayPane.listISObjects(lastClickedCell);
              break;
           case DisplayStatus.TEST_DATA:
           case DisplayStatus.SCTDB_DATA:
              guiControl.colorScalePane.setColorScale();
//              System.out.println("SelectorPane::TestData option setDisplayColors");
              guiControl.displayPane.setDisplayColors();   
              guiControl.displayPane.refreshDisplay();    
              guiControl.tablesDisplayPane.listTestResults();      
              break;
           default:
           }              
        }
    }


}
