/*
   UpdateOptionPopupMenu.java
*/
/**
   Popup menu allows GUI user to choose whether CalibrationController does updating of chip parameters
   based on test results.
   @author Alan Barr
   @date 13 october 2003
*/
package DisplayGUI;

import ipc.*;
import java.awt.*;
import java.awt.event.*;
import Sct_CalibrationController.CalibrationController;

public class UpdateOptionPopupMenu extends PopupMenu implements ItemListener 
{
    CheckboxMenuItem menuItemUpdate = new CheckboxMenuItem("Update");
    CheckboxMenuItem menuItemNoupdate = new CheckboxMenuItem("No update");
    CheckboxMenuItem menuItemWait = new CheckboxMenuItem("Wait");
    CalibrationControllerPanel2 ccPanel = null;

    public UpdateOptionPopupMenu(CalibrationControllerPanel2 ccPanel){
	this.ccPanel=ccPanel;
	add(menuItemUpdate);
	add(menuItemNoupdate);
	add(menuItemWait);
	
	menuItemUpdate.addItemListener(this);
	menuItemNoupdate.addItemListener(this);
	menuItemWait.addItemListener(this);
    }

    public void itemStateChanged (ItemEvent evt)
	{
	    try{
		Partition p = new Partition("SCT");
		org.omg.CORBA.Object ob = p.lookup(CalibrationController.instanceName, CalibrationController.serverName);
		if (ob == null) { 
		    System.out.println("CalibrationController not found.");
		}
		CalibrationController cc = Sct_CalibrationController.CalibrationControllerHelper.narrow(ob);
		Object target=evt.getSource();
		if ( evt.getStateChange() == ItemEvent.SELECTED ){
		    if (target == menuItemUpdate ) {
			cc.setUpdateOption(Sct_CalibrationController.CalibrationControllerPackage.UpdateOption.update);
		    } else if (target == menuItemNoupdate) {
			cc.setUpdateOption(Sct_CalibrationController.CalibrationControllerPackage.UpdateOption.noupdate);
		    } else if (target == menuItemWait) {
			cc.setUpdateOption(Sct_CalibrationController.CalibrationControllerPackage.UpdateOption._wait);
		    }
		} 
	    } catch (org.omg.CORBA.BAD_PARAM bp) {
		System.err.println("No CalibrationController object: " + bp);
	    } catch (org.omg.CORBA.COMM_FAILURE cf) {
		System.err.println("CORBA communications error: " + cf);
	    } catch (java.lang.Exception e) {
		System.err.println("Error" + e);
	    }
	}
}
