package ProdDatabase;
//
// @Author Dave Robinson
//
import java.io.*;
import java.util.Hashtable;
import java.util.Vector;
import Preferences.PreferencesInterface;
import guiUtilities.userPrompt;
import guiUtilities.DaveUtils;

public class SignoffDataInterface  {

   Hashtable data;
   public static final String[] keywords = {
"%SERIAL NUMBER",
"%CATEGORY",
"%COMMENT",
"%BARREL",
"%B56_REASON",
"%FAIL_REASON",
"%IV_CATEGORY",
"%IV_COMMENT",
"%ELECTRICAL_CATEGORY",
"%ELECTRICAL_COMMENT",
"%SCURVE_CATEGORY",
"%SCURVE_COMMENT"
};
   public static final String[] descriptions = {
"Serial Number",
"Overall category",
"Overall comment",
"Barrel type categorisation",
"Reason for B56 assignment",
"Reason for FAIL category",
"IV categorisation",
"Comment on IV performance",
"Electrical categorisation",
"Comment on electrical performance",
"S-curves categorsiation",
"Comment on S-curves"
};

   public static final int SERIALNO = 0;
   public static final int CATEGORY = 1;
   public static final int COMMENT  = 2;
   public static final int BARREL   = 3;
   public static final int B56REASON = 4;
   public static final int FAILREASON = 5;
   public static final int IVCATEGORY = 6;
   public static final int IVCOMMENT = 7;
   public static final int ELECTRICALCATEGORY = 8;
   public static final int ELECTRICALCOMMENT = 9;
   public static final int SCURVECATEGORY = 10;
   public static final int SCURVECOMMENT = 11;

   private String location;

   private static final boolean[][] compulsoryFields = {
             {true,false,false,false,false,false,true,true,true,true,true,true},    // require subset of parameters for institute upload
             {true,true,true,true,false,false,true,true,true,true,true,true}          // require every parameter present for signoff
             };

   public static final int INSTITUTE_UPLOAD=0;
   public static final int SIGNOFF_UPLOAD=1;

    public SignoffDataInterface(String serialNo, String location) {
            data = new Hashtable();
            this.location=location;
            try {
              String testno = GeneralUtilities.getSignoffDataTestNo(serialNo, location);
              Vector rawData = GeneralUtilities.getRawData(testno);
              if(rawData.size()>0) extractSignoffData((String)rawData.elementAt(1));
            }catch(Exception e){System.out.println("FAILED to download signoff data: "+e.toString());}
       }
    private void extractSignoffData(String dataString) {
       String[] lines = dataString.split("[\\n\\r]");
       String keyword="None";
       for(int line=0;line<lines.length;line++) {
           if(lines[line].startsWith("#")) continue;
           if(lines[line].startsWith("%")) {
             keyword=lines[line];
             continue;
             }
           if(data.containsKey(keyword)) {
               String oldData = (String)data.get(keyword);
               oldData += "\n"+lines[line];
               data.put(keyword,oldData);
               }
           else data.put(keyword,lines[line]);
           }
        }
    public String getString(int index) {
        if(data.containsKey(keywords[index])) return (String)data.get(keywords[index]);
        return null;
        }
    public void setParameter(int index, String value) {
        data.put(keywords[index],value);
        }
    public Vector allFieldsPresent(int uploadType) {
        Vector listOfMissingFields = new Vector();
        for(int i=0;i<keywords.length;i++) {
           if(compulsoryFields[uploadType][i] && !data.containsKey(keywords[i])) listOfMissingFields.add(descriptions[i]);
           }
        // if barrelcategory is '56', then require a reason
        if(data.containsKey(keywords[BARREL])) {
           if(((String)data.get(keywords[BARREL])).equals("56") && !data.containsKey(keywords[B56REASON])) listOfMissingFields.add(descriptions[B56REASON]);
           }
        // if category is 'FAIL' then require a reason
        if(data.containsKey(keywords[CATEGORY])) {
           if(((String)data.get(keywords[CATEGORY])).equals("FAIL") && !data.containsKey(keywords[FAILREASON])) listOfMissingFields.add(descriptions[FAILREASON]);
           }
        return listOfMissingFields;
        }

//**********************************************************************************
    public String writeUploadFiles(String sn) {
        String userInitials=null;
        try {
          userPrompt thisPrompt = new userPrompt(null,"","Your initials", GeneralUtilities.getNameList(PreferencesInterface.getInstance().getPreference(PreferencesInterface.INSTITUTE)));
          thisPrompt.pack();
//          thisPrompt.setLocationRelativeTo(topFrame);
          thisPrompt.setVisible(true);
         userInitials = thisPrompt.getValidatedText();
          if (userInitials.equals("None")) return null;
          } catch(Exception e1){System.out.println("ERROR retrieving list of users from SCTDB: "+e1.toString()); return null;}
        try {
          return writeUploadFile(sn,writeUploadDataFile(sn),userInitials);
          }catch(Exception e) {System.out.println("FAILED to create upload file: "+e.toString()); return null;}
        }
    private File writeUploadDataFile(String sn) throws Exception {
        File outFile = new File(PreferencesInterface.getInstance().getPreference(PreferencesInterface.UPLOAD_DIR),sn+"_signoffdata.txt");
        BufferedWriter out = new BufferedWriter(new FileWriter(outFile));
        for(int i=0;i<keywords.length;i++) {
            if(!data.containsKey(keywords[i])) continue;
            out.write(keywords[i]);
            out.newLine();
            out.write((String)data.get(keywords[i]));
            out.newLine();
            }
         out.close();
         return outFile;
         }
    private String writeUploadFile(String sn, File dataFile, String userInitials) throws Exception {
        PreferencesInterface prefs = PreferencesInterface.getInstance();
        File outFile = new File(prefs.getPreference(PreferencesInterface.UPLOAD_DIR),sn+"_signoff.upl");
        BufferedWriter out = new BufferedWriter(new FileWriter(outFile));
        out.write("%NEWTEST");
        out.newLine();
        out.write("SERIAL NUMBER : "+sn);
        out.newLine();
        out.write("TEST MADE BY : "+userInitials);
        out.newLine();
        out.write("LOCATION NAME : "+location);
        out.newLine();
        out.write("Run number : SIGNOFF");
        out.newLine();
        out.write("TEST DATE : "+DaveUtils.getTodaysDate());
        out.newLine();
        out.write("PASSED : YES");
        out.newLine();
        out.write("PROBLEM : NO");
        out.newLine();
        out.write("%bmMOD_RData");
        out.newLine();
        out.write("%TEST Rawdata");
        out.newLine();
        out.write("FILENAME : "+dataFile.getAbsolutePath());
        // note deliberately miss final newline, otherwise UploadTestData will fail!
        out.close();
        return outFile.getAbsolutePath();
        }
//*****************************************************************************
}